unit Bitbox;

{THE POINT:  To create checkbox group components that will take a byte or word
and provide dynamically sized boxes containing selected items from a universe
of 8 or 16 choices. Allowing the user to check/set bits via a form.}

{this unit takes advantage of delphi's small set implementation, which works
in bytes and words for sets with less than 9/17 members respectively.}

{CAVEAT: WHILE this all seems to work I wrote this because i needed it and I have not
yet come back to really really look it over. It's used in production but could use a few
more revisions before its 'library quality'; Also I'd like to make my initial tester for
it into a component-editor. All in good time.

{RELEASED BECAUSE: Someone thought this was important.}

{HOW TO USE:
  Register TBitBox and TdbBitBox and play with the properties.
  These/This is a passive component and is used to get/set/show values
  e.g. the input/output is the 'Numeric' property}

{------------------------------------------------------------------------------}
{                                                                              }
{------------------------------------------------------------------------------}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, StdCtrls;

Const
  DefaultLabelLength=25;
  DefaultHintLength=50;

type
  {define the word compatible set}
  T16Bits = (Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7
            ,Bit8,Bit9,BitA,BitB,BitC,BitD,BitE,BitF);
  TWordSet = Set of T16Bits;
  TWord = Record
    x:Word;
    end;

{------------------------------------------------------------------------------}
{ TBitBox  small set editor if you will                                        }
{------------------------------------------------------------------------------}

  {define a common groupbox for byte and word use}
  TBitBox = class(TGroupBox)
  private
    fMembers: TWordSet;
    fMask: TWordSet;
    fCaptions: TStringList;
    fOnChange: TNotifyEvent;
    fReadOnly: Boolean;
    constructor Create(aOwner:TComponent); Override;
    destructor Destroy; Override;
    procedure ChangeSelected(Sender:TObject); virtual;
  protected
    function GetMember:Word;
    procedure SetMember(Value:Word);
    procedure SetMembers(Value:TWordSet);
    procedure SetMask(Value:TWordSet);
    procedure SetCaptions(Value:TStringList);
  public
    procedure Loaded; Override;
    procedure Init; {[re]creates checkboxes from fUniverse/fMembers}
    procedure UpdateAll;
    procedure Update(Bit:T16Bits;aChecked:Boolean;aCaption:String); {updates chekbox}
  published
    property ReadOnly: Boolean read fReadOnly write fReadOnly default False;
    property Captions: TStringList read fCaptions write SetCaptions;
    property Possible: TWordSet read fMask write SetMask;
    property Selected: TWordSet read fMembers write SetMembers;
    property Numeric:  Word read GetMember write SetMember;
    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    end;

{------------------------------------------------------------------------------}
{ TdbBitBox  add a database access layer    NEEDS WORK.                        }
{------------------------------------------------------------------------------}

  TdbBitBox = class(TBitBox)
  private
    FDataLink: TFieldDataLink;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure ChangeSelected(Sender:TObject); override;
    procedure UpdateData(Sender: TObject);
    procedure EditingChange(Sender: TObject);
  protected
    function GetReadOnly: Boolean;
    procedure SetReadOnly(Value: Boolean);
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(aOwner:TComponent); Override;
    destructor Destroy; Override;
    property Field: TField read GetField;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    end;


implementation

{------------------------------------------------------------------------------}
{ CREATE, PREPARE AND DISPOSE OF THE COMPONENT                                 }
{------------------------------------------------------------------------------}

constructor TBitBox.Create(aOwner:TComponent);
var
  Bit: T16Bits;
begin
  inherited Create(aOwner);
  fMask:=[];
  fCaptions:=TStringList.Create;
  for Bit:= Bit0 to BitF do
    fCaptions.Add('(Bit'+inttoStr(ord(bit))+')');
end;

procedure TBitBox.Loaded;
begin
  inherited Loaded;
  Init;
end;

destructor TBitBox.Destroy;
begin
  fCaptions.Free;
  fCaptions:=nil;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
{ GET AND SET THE COMPONENT SPECIFIC PROPERTIES                                }
{------------------------------------------------------------------------------}

function TBitBox.GetMember:Word;
var
  s:TWordSet;
begin            {if are weird if you think this isn't weird. <G>}
  s:=fMembers;
  result:=tWord(s).x;
end;

procedure TBitBox.SetMember(Value:Word);
var
  s:TWordSet;
begin                     {why don't I make any attempt to explain this?}
  s:=TWordSet(Value);
  SetMembers(s);
end;

procedure TBitBox.SetMembers(Value:TWordSet);
begin
  if fMembers<>Value then begin
    fMembers:=Value;
    UpdateAll;              {lazy, but it runs on a pentium<g>}
    end;
end;

procedure TBitBox.SetMask(Value:TWordSet);
begin
  if fMask<>Value then begin
    fMask:=Value;
    init;
    end;
end;

procedure TBitBox.SetCaptions(Value:TStringList);
begin
  if fCaptions<>Value then begin
    fCaptions.Assign(Value);
    UpdateAll;                   {looks like a bad habit, but I told you it needed work.}
    end;
end;

{------------------------------------------------------------------------------}
{ INITIALIZE AND UPDATE ALL OR ONE CHECKBOX IN THE GROUP                       }
{------------------------------------------------------------------------------}

procedure TBitBox.Init;
var
  i,n:integer;
  Bit:T16Bits;
  c:TCheckBox;
begin
  if (csLoading in ComponentState) then
    exit;
  n:=ControlCount-1;
  if n>-1 then
    for i:=0 to n do
      Controls[0].Free;
  n:=0;
  for bit:= Bit0 to BitF do
    if bit in fMask then begin
      c:=TCheckBox.Create(self);              {make 'sub components}
      with c do begin
        Tag:=ord(bit);
        Caption:= fCaptions.Strings[Tag];
        Checked:= bit in fMembers;
        Parent:=Self;
        OnClick:=ChangeSelected;                {must surface events better!! pass-out!}
        Left:=10;
        Width:=Self.Width-Left-5;
        n:=n+1;
        Top:=n*20;
        end;
      end;
  Height:=(n+1)*20;
end;

procedure TBitBox.UpdateAll;
var
  Bit:T16Bits;
begin
  for Bit:= Bit0 to BitF do
    if Bit in fMask then
      Update(Bit,Bit in fMembers,fCaptions.Strings[ord(Bit)])
end;

procedure TBitBox.Update(Bit:T16Bits;aChecked:Boolean;aCaption:String);
var
  i,n:integer;
begin
  n:=ControlCount-1;
  if n>-1 then
    for i:=0 to n do
      if Controls[i].Tag=ord(Bit) then
        with TCheckBox(Controls[i]) do begin
          Caption:=aCaption;
          Checked:=aChecked;
          break;
          end;
end;

{------------------------------------------------------------------------------}
{ PROCS FOR CUSTOM EVENTS                                                      }
{------------------------------------------------------------------------------}

procedure TBitBox.ChangeSelected(Sender:TObject);
var
  c:TCheckBox;
  b:T16Bits;
begin
  c:=TCheckBox(Sender);
  b:=T16Bits(c.Tag);
  if c.Checked then
    fMembers:=fMembers+[b]
  else
    fMembers:=fMembers-[b];
  if assigned(fOnChange) then
    fOnchange(Sender);
end;

{------------------------------------------------------------------------------}
{ PROCS TO ERECT THE DATASOURCE CONNECTED COMPONENT                            }
{------------------------------------------------------------------------------}

constructor TdbBitBox.Create(aOwner:TComponent);            { I Love Delphi! }
begin
  inherited Create(aOwner);
  inherited ReadOnly := True;
  FDataLink:= TFieldDataLink.Create;
  FDataLink.OnDataChange:= DataChange;
  FDataLink.Control := Self;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TdbBitBox.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  fCaptions.Free;
  fCaptions:=nil;
  inherited Destroy;
end;

procedure TdbBitBox.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

{------------------------------------------------------------------------------}
{ PLUMBING AND READ-ONLY                                                       }
{------------------------------------------------------------------------------}

function TdbBitBox.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TdbBitBox.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

function TdbBitBox.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TdbBitBox.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TdbBitBox.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TdbBitBox.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TdbBitBox.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

{------------------------------------------------------------------------------}
{                                                                              }
{------------------------------------------------------------------------------}

procedure TdbBitBox.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    Numeric := FDataLink.Field.AsInteger              {**MUST EXTEND TO USE Alpha 1's}
  else
    if csDesigning in ComponentState then Numeric := 0; {REALLY OUGHT TO WORK WITH CHARS!}
end;                                  {Someone please send me a TOneByteField Descendant!}

procedure TdbBitBox.ChangeSelected(Sender:TObject);
begin
  inherited ChangeSelected(Sender);
  if FDataLink.Field <> nil then
    if not (csDesigning in ComponentState) then
      UpdateData(Sender);
end;

procedure TdbBitBox.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
end;

procedure TdbBitBox.UpdateData(Sender: TObject);
begin
  if Numeric<>FDataLink.Field.AsInteger then         {same thing.. will figure it out!}
    if FDataLink.Edit then                 {perhaps there will be another public release}
      FDataLink.Field.AsInteger:= Numeric;
end;


{------------------------------------------------------------------------------}
{                                                                              }
{------------------------------------------------------------------------------}

end.
