unit Copier;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TCopier = class(TComponent)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }
    procedure CopyFile(Source, Dest: String);
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TCopier]);
end;

{This is a modified version of the file copying procedure in the Delphi
 "File Manager" example}
procedure ICopyFile(FileName, DestName: Integer);
var
  CopyBuffer: Pointer;
  TimeStamp, BytesCopied: Longint;
  Source, Dest: Integer;
  Destination: TFileName;
const
  ChunkSize: Longint = 8192;
begin
  GetMem(CopyBuffer, ChunkSize);
  try
    Source:=FileName;
    if Source < 0 then ShowMessage('Error in source file/filename!');
    try
      Dest:=DestName;
      if Dest < 0 then ShowMessage('Error in destination file/filename!');
      try
        repeat
          BytesCopied := FileRead(Source, CopyBuffer^, ChunkSize); 
          if BytesCopied > 0 then 
            FileWrite(Dest, CopyBuffer^, BytesCopied); 
        until BytesCopied < ChunkSize; 
      finally
        FileClose(Dest);
      end;
    finally
      FileClose(Source); 
    end;
  finally
    FreeMem(CopyBuffer, ChunkSize);
  end;
end;

procedure TCopier.CopyFile(Source, Dest: String);
var
  {The ICopyFile procedure takes integer handles to the files.}
  ISource, IDest: Integer;

  {Error-handling checks if the source file exists before copying.}
  SourceExists: Boolean;
begin
  {Initialize Local Variables.}
  ISource:=0;
  IDest:=0;
  SourceExists:=False;

  {Does the source file exist?}
  SourceExists:=FileExists(ExpandFilename(Source));

  {If not, give an error!}
  If not(SourceExists) then MessageDlg('Source file '+ExpandFileName(Source)+
                            ' does not exist!', mtError, [mbOk], 0);

  {If it does, go ahead and copy the file.}
  If SourceExists then begin
    {Get integer handles for both files, to read and write to both.}
    ISource := FileOpen(Source, fmOpenReadWrite);
    IDest := FileCreate(Dest);

    {Call the ICopy procedure to copy the files.}
    ICopyFile(ISource, IDest);
  end;
end;

end.
