unit Callback;
{$T+}

{ This unit enumerates Top level windows searching for your
  application already running. }

interface

uses WinTypes, WinProcs, SysUtils, Messages, Forms;

{ Creating an Object to store Find Window data wasn't necessary,
  a record would have done fine. However, I love Delphi's Object
  creation ease, and this object demonstrates how to use an
  Initialization section in any Unit and properly destroy these
  dynamically created Objects when multiple instances of an
  application are Halted. }

type
  TFindWnd = Class(TObject)
    WindowClass: PChar;
    WindowText: PChar;
    doClass: boolean;
    doText: boolean;
    Success: boolean;
    Handle: THandle;
    constructor Create;
    destructor Destroy; override;
    end;

function AlreadyRunning: boolean;
procedure Free_FindWnd;
{$ifdef WIN32}
function CallBackProc(hwnd: Word; lParam: Longint): boolean; stdcall; export;
{$else}
function CallBackProc(hwnd: Word; lParam: Longint): boolean; export;
{$endif}

var
  FindWnd: TFindWnd;

implementation

uses Run1Time;

{ Declare Local Variables }

const
  lResult: Longint=0;

var
  Buf: PChar;

function AlreadyRunning: boolean;
  begin
    Result := False;
    with FindWnd do begin
      Handle := 0;
      doClass := True;
      doText := True;
      WindowClass := pAppClass;
      WindowText := pCaption;
      end;
    { Child Windows can be located by using the following
      function call with API "EnumChildWindows" instead of
      "EnumWindows".

      EnumWindows is used instead of API FindWindow so that
      16 bit application instances can find 32 bit application
      instances of Application.MainForm. }
    EnumWindows(@CallBackProc, 0);
    Result := (FindWnd.Handle > 0);
    if Result then PrevMyForm := FindWnd.Handle;
    end;

function CallBackProc(hwnd: Word; lParam: Longint): boolean;
  var
    Success: boolean;
  begin
    Result := True;
    Success := False;
    FillChar(szBuf, SizeOf(szBuf), 0);
    if FindWnd.doClass then begin
      lResult := GetClassName(hWnd, Buf, Pred(SizeOf(szBuf)));
      if (lResult > 0) and (StrLen(Buf) > 0) then
        if StrLIComp(FindWnd.WindowClass, Buf, StrLen(FindWnd.WindowClass)) = 0 then
          begin
          if not FindWnd.doText then begin
            FindWnd.Handle := hwnd;
            { A quick exit, stop enumeration by returning False }
            Result := False;
            end;
          Success := True;
          end
        else exit;
        end;
    if FindWnd.doText then begin
      FillChar(szBuf, SizeOf(szBuf), 0);
      lResult := SendMessage(hWnd, WM_GETTEXT, Pred(SizeOf(szBuf)), Longint(@szBuf));
      if (lResult > 0) and (StrLen(Buf) > 0) then
        if StrLIComp(FindWnd.WindowText, Buf, StrLen(FindWnd.WindowText)) = 0 then
          begin
          FindWnd.Handle := hwnd;
          { A quick exit, stop enumeration by returning False }
          Result := False;
          end
        else Success := False;
      end;
    end;

{ Declare an exit procedure }
procedure Free_FindWnd;
  begin
    FindWnd.Free;
    end;

constructor TFindWnd.Create;
  begin
    inherited Create;
    end;

destructor TFindWnd.Destroy;
  begin
    inherited Destroy;
    end;

Initialization

  Buf := szBuf;
  FindWnd := TFindWnd.Create;
  { Exit Proc's are called when your application calls "Halt".
    Here we quickly destroy dynamically created FindWnd Object
    if an instance of this application is found. Delphi 2.0
    has a "Finalize" section that would accomplish the same
    effect, however, AddExitProc is an API function that remains
    constant for both 16 and 32 bit applications. }
  if FindWnd <> nil then AddExitProc(Free_FindWnd);

end.
