unit ODBC;

{ TODBC: ODBC object 5/22/95}
{ by Jeff Atwood, JAtwood159@AOL.COM }
{ }
{ IMPORTANT: This unit declares many of the functions in the ODBC libraries,   }
{ but I HAVE NOT checked any of them for accuracy. PLEASE check the parameters }
{ for any ODBC call that I am not already using before you use it the first    }
{ time. I translated these from VB, and the mind is strong, but the fingers    }
{ are weak.. :) The original VB.BAS declaration file is included here, too..   }
{ }
{ For details on how to use this object, see the code in the demo form. Right  }
{ now the object just allows you to Connect to the ODBC source, send SQL of    }
{ any kind to the ODBC Connection, and fill a TStringGrid with the result set  }
{ (if any). It's virtually certain that I will be extending the functionality  }
{ of this control with the help of KangaDru@AOL.COM... also if any enterprising}
{ individuals want to add functionality, please do so!! You'll need the ODBC   }
{ SDK for documentation on the ODBC functions, though. You can get this from   }
{ Microsoft for some nominal fee..}
{ }
{ Any rants, raves, comments, improvements, or rutabagas, please E-Mail me at  }
{ JAtwood159@AOL.COM!! Have Fun! }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Dialogs, Grids;

Const

{ generally useful constants }

  SQL_NTS = -3;                  {  NTS = Null Terminated String}
  SQL_SQLSTATE_SIZE = 5;         {  size of SQLSTATE}
  SQL_MAX_MESSAGE_LENGTH = 512;  {  message buffer size}
  SQL_MAX_DSN_LENGTH = 32;       {  maximum data source name size}

{ RETCODEs }

  SQL_ERROR = -1;
  SQL_INVALID_HANDLE = -2;
  SQL_NO_DATA_FOUND = 100;
  SQL_SUCCESS = 0;
  SQL_SUCCESS_WITH_INFO = 1;

{ SQLFreeStmt defines }

  SQL_CLOSE = 0;
  SQL_DROP = 1;
  SQL_UNBIND = 2;
  SQL_RESET_PARAMS = 3;

{ SQLSetParam defines }

  SQL_C_DEFAULT = 99;

{ SQLTransact defines }

  SQL_COMMIT = 0;
  SQL_ROLLBACK = 1;

{ Standard SQL datatypes, using ANSI type numbering }

  SQL_CHAR = 1;
  SQL_NUMERIC = 2;
  SQL_DECIMAL = 3;
  SQL_INTEGER = 4;
  SQL_SMALLINT = 5;
  SQL_FLOAT = 6;
  SQL_REAL = 7;
  SQL_DOUBLE = 8;
  SQL_VARCHAR = 12;

  SQL_TYPE_MIN = 1;
  SQL_TYPE_NULL = 0;
  SQL_TYPE_MAX = 12;

{ C datatype to SQL datatype mapping    SQL types }

  SQL_C_CHAR = SQL_CHAR;         {  CHAR, VARCHAR, DECIMAL, NUMERIC}
  SQL_C_LONG = SQL_INTEGER;      {  INTEGER}
  SQL_C_SHORT = SQL_SMALLINT;    {  SMALLINT}
  SQL_C_FLOAT = SQL_REAL;        {  REAL}
  SQL_C_DOUBLE = SQL_DOUBLE;     {  FLOAT, DOUBLE}

{ NULL status constants.  These are used in SQLColumns, SQLColAttributes,
  SQLDescribeCol, and SQLSpecialColumns to describe the nullablity of a
  column in a table.  SQL_NULLABLE_UNKNOWN can be returned only by
  SQLDescribeCol or SQLColAttributes.  It is used when the DBMS's meta-data
  does not contain this info. }

  SQL_NO_NULLS = 0;
  SQL_NULLABLE = 1;
  SQL_NULLABLE_UNKNOWN = 2;

{ Special length values }

  SQL_NULL_DATA = -1;
  SQL_DATA_AT_EXEC = -2;

{ SQLColAttributes defines }

  SQL_COLUMN_COUNT = 0;
  SQL_COLUMN_NAME = 1;
  SQL_COLUMN_TYPE = 2;
  SQL_COLUMN_LENGTH = 3;
  SQL_COLUMN_PRECISION = 4;
  SQL_COLUMN_SCALE = 5;
  SQL_COLUMN_DISPLAY_SIZE = 6;
  SQL_COLUMN_NULLABLE = 7;
  SQL_COLUMN_UNSIGNED = 8;
  SQL_COLUMN_MONEY = 9;
  SQL_COLUMN_UPDATABLE = 10;
  SQL_COLUMN_AUTO_INCREMENT = 11;
  SQL_COLUMN_CASE_SENSITIVE = 12;
  SQL_COLUMN_SEARCHABLE = 13;
  SQL_COLUMN_TYPE_NAME = 14;
{xIFDEF ODBC 2.00}
  SQL_COLUMN_TABLE_NAME = 15;
  SQL_COLUMN_OWNER_NAME = 16;
  SQL_COLUMN_QUALIFIER_NAME = 17;
  SQL_COLUMN_LABEL = 18;
{xENDIF ODBC 2.00}
  SQL_COLATT_OPT_MAX = SQL_COLUMN_LABEL;

{ SQLColAttributes subdefines for SQL_COLUMN_UPDATABLE }

 SQL_ATTR_READONLY = 0;
 SQL_ATTR_WRITE = 1;
 SQL_ATTR_READWRITE_UNKNOWN = 2;

{ SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE
  These are also used by SQLGetInfo }

 SQL_UNSEARCHABLE = 0;
 SQL_LIKE_ONLY = 1;
 SQL_ALL_EXCEPT_LIKE = 2;
 SQL_SEARCHABLE = 3;

{  SQLError defines  }

 SQL_NULL_HENV = 0;
 SQL_NULL_HDBC = 0;
 SQL_NULL_HSTMT = 0;

{ Options for SQLDriverConnect }

 SQL_DRIVER_NOPROMPT = 0;
 SQL_DRIVER_COMPLETE = 1;
 SQL_DRIVER_PROMPT = 2;
 SQL_DRIVER_COMPLETE_REQUIRED = 3;

{ ODBC Core Definitions }

Function SQLAllocConnect(henv: Longint; var phdbc: Longint): Integer;
Function SQLAllocEnv(var phenv: Longint): Integer;
Function SQLAllocStmt(hdbc: Longint; var phstmt: Longint): Integer;
Function SQLBindCol(hstmt: Longint; icol: integer; fCType: integer;
                    rgbValue: Pointer; cbValueMax: Longint;
                    pcbValue: Longint) : Integer;
Function SQLCancel(hstmt: Longint): Integer;
Function SQLColAttributes(hstmt: Longint; icol: integer;
                          fDescType: integer; rgbDesc: Pointer;
                          cbDescMax: integer; var pcbDesc: integer;
                          var pfDesc: Longint): Integer;
Function SQLConnect(hdbc: Longint; szDSN: String; cbDSN: integer;
                    szUID: PChar; cbUID: integer; szAuthStr: PChar;
                    cbAuthStr: integer): Integer;
Function SQLDescribeCol(hstmt: Longint; icol: integer; SzColName: Pchar;
                        cbColNameMax: integer; var pcbColName: integer;
                        var pfSqlType: integer; var pcbColDef: Longint;
                        var pibScale: integer; var pfNullable: integer): Integer;
Function SQLDisconnect(hdbc: Longint): Integer;
Function SQLError(henv: Longint; hdbc: Longint; hstmt: Longint;
                  szSqlState: Pchar; var pfNativeError: Longint; szErrorMsg: PChar;
                  cbErrorMsgMax: integer; var pcbErrorMsg: integer): Integer;
Function SQLExecDirect(hstmt: Longint; szSqlStr: PChar;
                       cbSqlStr: Longint): Integer;
Function SQLExecute(hstmt: Longint): Integer;
Function SQLFetch(hstmt: Longint): Integer;
Function SQLFreeConnect(hdbc: Longint): Integer;
Function SQLFreeEnv(henv: Longint): Integer;
Function SQLFreeStmt(hstmt: Longint; fOption: integer): Integer;
Function SQLGetCursorName(hstmt: Longint; szCursor: Pchar;
                          cbCursorMax: integer; pcbCursor: integer): Integer;
Function SQLNumResultCols(hstmt: Longint; var pccol: integer): Integer;
Function SQLPrepare(hstmt: Longint; szSqlStr: Pchar;
                    cbSqlStr: Longint): Integer;
Function SQLRowCount(hstmt: Longint; var pcrow: Longint): Integer;
Function SQLSetCursorName(hstmt: Longint; szCursor: PChar;
                          cbCursor: integer): Integer;
Function SQLSetParam(hstmt: Longint; ipar: integer; fCType: integer;
                     fSqlType: integer; cbColDef: Longint;
                     ibScale: integer; rgbValue: Pointer;
                     pcbValue: Longint): Integer;
Function SQLTransact(henv: Longint; hdbc: Longint;
                     fType: integer): Integer;


{ ODBC Level 1 }

Function SQLBindParameter(hstmt: longint; ipar: Integer; fParamType: Integer;
                          fCType: Integer; fSqlType: Integer; cbColDef: longint;
                          ibScale: Integer; rgbValue : Pointer; cbValueMax: longint;
                          pcbValue: Longint): Integer;
Function SQLColumns(hstmt: longint; szTblQualifier : Pointer; cbTblQualifier: Integer;
                    szTblOwner : Pointer; cbTblOwner: Integer; szTblName : Pointer;
                    cbTblName: Integer; szColName : Pointer; cbColName: Integer): Integer;
Function SQLDriverConnect(hdbc: longint; hwnd: Integer; szCSIn: PChar; cbCSIn: Integer;
                          szCSOut: PChar; cbCSMax: Integer; var cbCSOut: Integer;
                          fDrvrComp: Integer): Integer;
Function SQLGetConnectOption(hdbc: longint; fOption: Integer; pvParam : Pointer): Integer;
Function SQLGetData(hstmt: longint; icol: Integer; fCType: Integer; rgbValue: Pointer;
                    cbValueMax: longint; var pcbValue: longint): Integer;
Function SQLGetFunctions(hdbc: longint; fFunction: Integer; var pfExists: Integer): Integer;
Function SQLGetInfo(hdbc: longint; fInfoType: Integer; rgbInfoValue: Pointer; var
                    cbInfoMax: Integer; var cbInfoOut: Integer): Integer;
Function SQLGetStmtOption(hstmt: longint; fOption: Integer; pvParam: Pointer): Integer;
Function SQLGetTypeInfo(hstmt: longint; fSqlType: Integer): Integer;
Function SQLParamData(hstmt: longint; prgbValue : Pointer): Integer;
Function SQLPutData(hstmt: longint; rgbValue : Pointer; cbValue: longint): Integer;
Function SQLSetConnectOption(hdbc: longint; fOption: Integer; vParam: longint): Integer;
Function SQLSetStmtOption(hstmt: longint; fOption: Integer; vParam: longint): Integer;
Function SQLSpecialColumns(hstmt: longint; fColType: Integer; szTblQualifier: Pointer;
                           cbTblQualifier: Integer; szTblOwner: Pointer;
                           cbTblOwner: Integer; szTblName: Pointer;
                           cbTblName: Integer; fScope: Integer;
                           fNullable: Integer): Integer;
Function SQLStatistics(hstmt: longint; szTblQualifier: Pointer; cbTblQualifier: Integer;
                       szTblOwner : Pointer; cbTblOwner: Integer; szTblName: Pointer;
                       cbTblName: Integer; fUnique: Integer; fAccuracy: Integer): Integer;
Function SQLTables(hstmt: longint; szTblQualifier : Pointer; cbTblQualifier: Integer;
                   szTblOwner : Pointer; cbTblOwner: Integer; szTblName: Pointer;
                   cbTblName: Integer; szTblType : Pointer; cbTblType: Integer): Integer;


{ The master TODBC object }

Type TODBC = Class(TObject)
  private
    henv: Longint;
    hdbc: Longint;
    hstmt: Longint;
    verbose: Boolean;
    procedure attempt(ResultCode: Integer; Action: PChar);
    procedure clearHandles;
    procedure describeError;
    function RunQuery(tables: Boolean; var ReturnedData: Boolean): Boolean;
  public
    ConnectString: PChar;
    Initialized: Boolean;
    SQL: PChar;
    constructor create;
    destructor destroy; override;
    procedure Init(hwnd: Integer);
    procedure FillGrid(anygrid: TStringGrid);
    function Query(var ReturnedData: Boolean): Boolean;
    function GetTables: Boolean;
end;


implementation

{ ODBC Core }
Function SQLAllocConnect; external 'ODBC';
Function SQLAllocEnv; external 'ODBC';
Function SQLAllocStmt; external 'ODBC';
Function SQLBindCol; external 'ODBC';
Function SQLCancel; external 'ODBC';
Function SQLColAttributes; external 'ODBC';
Function SQLConnect; external 'ODBC';
Function SQLDescribeCol; external 'ODBC';
Function SQLDisconnect; external 'ODBC';
Function SQLError; external 'ODBC';
Function SQLExecDirect; external 'ODBC';
Function SQLExecute; external 'ODBC';
Function SQLFetch; external 'ODBC';
Function SQLFreeConnect; external 'ODBC';
Function SQLFreeEnv; external 'ODBC';
Function SQLFreeStmt; external 'ODBC';
Function SQLGetCursorName; external 'ODBC';
Function SQLNumResultCols; external 'ODBC';
Function SQLPrepare; external 'ODBC';
Function SQLRowCount; external 'ODBC';
Function SQLSetCursorName; external 'ODBC';
Function SQLSetParam; external 'ODBC';
Function SQLTransact; external 'ODBC';

{ ODBC Level 1}
Function SQLBindParameter; external 'ODBC';
Function SQLColumns; external 'ODBC';
Function SQLDriverConnect; external 'ODBC';
Function SQLGetConnectOption; external 'ODBC';
Function SQLGetData; external 'ODBC';
Function SQLGetFunctions; external 'ODBC';
Function SQLGetInfo; external 'ODBC';
Function SQLGetStmtOption; external 'ODBC';
Function SQLGetTypeInfo; external 'ODBC';
Function SQLParamData; external 'ODBC';
Function SQLPutData; external 'ODBC';
Function SQLSetConnectOption; external 'ODBC';
Function SQLSetStmtOption; external 'ODBC';
Function SQLSpecialColumns; external 'ODBC';
Function SQLStatistics; external 'ODBC';
Function SQLTables; external 'ODBC';

{ set up the object, allocate string space }
constructor TODBC.Create;
begin
  inherited create;
  initialized := False;
  henv := 0;
  hdbc := 0;
  hstmt := 0;
  verbose := False;
  connectString := strAlloc(SQL_MAX_MESSAGE_LENGTH);
  SQL := strAlloc(2048);
end;

{ closes any handles in use by TODBC }
procedure TODBC.ClearHandles;
begin
  if hstmt <> 0 then begin
    attempt(SQLFreeStmt(hstmt, SQL_CLOSE), 'free statement handle');
    hstmt := 0;
  end;
  if hdbc <> 0 then begin
    attempt(SQLDisconnect(hdbc), 'close connection');
    attempt(SQLFreeConnect(hdbc), 'free connection handle');
    hdbc := 0;
  end;
  if henv <> 0 then begin
    attempt(SQLFreeEnv(henv), 'free environment handle');
    henv := 0;
  end;
end;

{ basically just deallocate strings }
destructor TODBC.Destroy;
begin
  clearHandles;
  strDispose(connectString);
  strDispose(SQL);
end;


{ this is the 'wrapper' that handles error codes }
procedure TODBC.Attempt(ResultCode: Integer; Action: PChar);
var
  retval: Integer;
begin
  if (ResultCode <> SQL_SUCCESS) then begin
    retval := MessageBox(0, Action, 'ODBC Driver Error', MB_ICONSTOP);
    end
  else
    if verbose then
      retval := MessageBox(0, Action, 'ODBC action completed:', 0);
end;

{ enumerates errors by asking ODBC what the error was }
procedure TODBC.DescribeError;
var
  sqlState: PChar;
  errMsg: PChar;
  Outlen: Integer;
  ErrCode: Longint;
  retval: Integer;
  retval2: Integer;
begin
  errCode := 0;
  sqlState := strAlloc(SQL_SQLSTATE_SIZE);
  errMsg := strAlloc(SQL_MAX_MESSAGE_LENGTH);
  repeat
    retval := SQLError(henv, hdbc, hstmt, sqlState, errCode, errMsg,
                       SQL_MAX_MESSAGE_LENGTH - 1, Outlen);
    if ((retval = SQL_SUCCESS) Or (retval = SQL_SUCCESS_WITH_INFO)) then begin
      if Outlen = 0 then begin
        retval2 := MessageBox(0, 'No error information available', 'ODBC Error', MB_ICONSTOP);
        end
      else begin
        retval2 := MessageBox(0, errMsg, 'ODBC Error', MB_ICONSTOP);
      end;
    end;
  until retval <> SQL_SUCCESS;
  strDispose(sqlState);
  strDispose(errMsg);
end;


{ open an ODBC connection. Uses the ConnectString }
{ property of TODBC. Will work with any connect string }
{ and prompt if missing. Provide valid connect string }
{ to avoid prompt }
procedure TODBC.Init(hwnd: Integer);
var
  retval: Integer;
  output: Integer;
begin

  { clean out any existing handles }
  clearHandles;

  { get ODBC environment handle }
  attempt(SQLAllocEnv(henv), 'allocate ODBC environment handle');

  { get a connection handle }
  attempt(SQLAllocConnect(henv, hdbc), 'allocate ODBC connection handle');

  { open the connection using the handle }
  retval := SQLDriverConnect(hdbc, hwnd, ConnectString, strLen(ConnectString),
                             ConnectString, SQL_MAX_MESSAGE_LENGTH,
                             output, SQL_DRIVER_COMPLETE);

  if (retval = SQL_ERROR) or (retval = SQL_INVALID_HANDLE) or (retval = SQL_NO_DATA_FOUND) then begin
    attempt(SQLFreeConnect(hdbc), 'free connection handle');
    hdbc := 0;
    describeError;
    initialized := False;
    exit;
  end;

  initialized := True;

end;


{ Internal function to send SQL through ODBC }
function TODBC.RunQuery(tables: Boolean; var ReturnedData: Boolean): Boolean;
var
  retval : Integer;
  cols: Integer;
begin

  { make sure ODBC is initialized first }
  if not initialized then begin
    retval := MessageBox(0, 'ODBC object not initialized.', 'Query',
                         MB_ICONEXCLAMATION);
    result := False;
    exit;
  end;

  { make sure we have a NEW statement handle first }
  if hstmt <> 0 then begin
    attempt(SQLFreeStmt(hstmt, SQL_CLOSE), 'free statement handle');
    hstmt := 0;
  end;

  attempt(SQLAllocStmt(hdbc, hstmt), 'allocate statment handle');

  if hstmt = 0 then begin
    result := False;
    exit;
  end;

  { make sure we have an SQL string to process }
  if ((not tables) and (strLen(SQL) = 0)) then begin
      retval := MessageBox(0, 'SQL string is empty.', 'Query',
                           MB_ICONEXCLAMATION);
      result := False;
      exit;
  end;

  { run the query or table query }
  if tables then
    retval := SQLTables(hstmt, nil, 0, nil, 0, nil, 0, nil, 0)
  else
    retval := SQLExecDirect(hstmt, SQL, strLen(SQL));

  if retval <> SQL_SUCCESS then begin
    DescribeError;
    result := False;
    exit;
  end;

  { see if query returned data }
  retval := SQLNumResultCols(hstmt, cols);
  if retval <> SQL_SUCCESS then begin
    DescribeError;
    result := False;
    ReturnedData := False;
    exit;
  end;
  if cols = 0 then
    ReturnedData := False
  else
    ReturnedData := True;

  result := True;

end;


{ Wrapper for RunQuery, uses SQL }
function TODBC.Query(var ReturnedData: Boolean): Boolean;
begin
  result := RunQuery(False, ReturnedData);
end;

{ Wrapper for RunQuery, gets Table list }
function TODBC.GetTables: Boolean;
var
  returnedData: Boolean;
begin
  result := RunQuery(True, returnedData);
end;

{ Fills TStringGrid with data, if any, on ODBC }
{ connection. You can use the ReturnedData property }
{ to see if there is data before doing this. Not all }
{ SQL returns data, of course. }
procedure TODBC.FillGrid(anygrid: TStringGrid);
var
  cols: integer;
  retval: integer;
  fetchval: integer;
  cnt: integer;
  maxwidth: integer;
  fvalue: PChar;
  Outlen: integer;
  coltypecode: integer;
  maxcolwidth: longint;
  colscale: integer;
  colnullable: integer;
  currow: Integer;
  pcbvalue: longint;
begin

  { get number of columns in result set }
  retval := SQLNumResultCols(hstmt, cols);
  if retval <> SQL_SUCCESS then begin
    DescribeError;
    exit;
  end;
  if cols = 0 then begin
    retval := MessageBox(0, 'Query contains no rows.', 'FillGrid', MB_ICONINFORMATION);
    exit;
  end;

  { set grid size appropriately }
  anygrid.RowCount := 1;
  anygrid.ColCount := cols + 1;
  anygrid.ColWidths[0] := 32;

  fValue := StrAlloc(SQL_MAX_MESSAGE_LENGTH);

  { set column titles and column widths}
  For cnt := 1 to cols do begin
    Attempt(SQLDescribeCol(hstmt, cnt, fValue, SQL_MAX_MESSAGE_LENGTH - 1, Outlen,
            ColTypeCode, maxColWidth, ColScale, ColNullable), 'DescribeCol');
    anygrid.cells[cnt, 0] := strPas(fValue);
    if maxColWidth > 100 then
      anyGrid.ColWidths[cnt] := 256
    else
      anygrid.ColWidths[cnt] := maxColWidth * 10;
  end;

  currow := 0;

  repeat

    { fetch rows }
    fetchval := SQLFetch(hstmt);

    { exit infinite loop when hit error or eof}
    if fetchval <> SQL_SUCCESS then begin
      break;
    end;

    { increment rows }
    currow := currow + 1;
    anygrid.RowCount := currow;
    anygrid.cells[0,currow] := inttostr(currow);

    { get column data }
    for cnt := 1 to cols do begin
      retval := SQLGetData(hstmt, cnt, SQL_C_CHAR, fValue,
                           SQL_MAX_MESSAGE_LENGTH, pcbValue);
      if retval <> SQL_SUCCESS then begin
        DescribeError;
      end;
      if retval <> SQL_ERROR then begin
        if pcbValue = SQL_NULL_DATA then begin
          anygrid.cells[cnt, currow] := '(null)';
          end
        else begin
          anygrid.cells[cnt, currow] := strPas(fValue);
        end;
      end;
    end;

  until False;

  If anygrid.rowcount > 1 then begin
    anygrid.FixedCols := 1;
    anygrid.FixedRows := 1;
  end;

  strDispose(fValue);

end;

end.
