unit Run1time;
{$T+}

interface

uses WinTypes, WinProcs, SysUtils, Classes, Forms, Controls, Messages;

{ Declare Public variables }
const
  { pCaption must be set at run time. If your Delphi 2.0 design
    interface has a MainForm.Caption property (Object Inspector)
    identical to this run time pCaption declaration then Run1Time
    will not behave properly. }

  pCaption : PChar= 'Run1Time' + #0;

  { pAppClass must be set at design time and must be identical
   to your Type declaration for your TForm(MainForm).ClassName or
   Run1Time will not behave properly.

   Example: TMyForm = class(TForm)

   Please change this pAppClass declaration and your
   TForm(MainForm).ClassName to any unique Class Name for
   your application or we may have a bunch of applications
   on the market that do not behave properly because too many
   app's have a MainForm.ClassName of "TMyForm". }

  pAppClass: PChar= 'TMyForm' + #0;

  { Custom Windows Messages }

  hParamMsg: Word =0;
  hShowWnd: Word =0;

  { Additional Global variable }
  PrevMyForm: Word=0;

type
  TMyForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    { CreateParams allows you to set any Delphi forms'
      Parent property to any external applications' window.
      And it allows you to set any TParams at run time. }
    procedure CreateParams( var Params: TCreateParams ); override;
  private
    { Private declarations }
  public
    { Public declarations }
  protected
    { I like to override Delphi's WndProc for custom messages.
      This allows me to get a hold of these messages before Delphi's
      runtime functions. }
    procedure WndProc(var Msg: TMessage); override;
  end;

var
  MyForm: TMyForm;
  { General Buffer for API calls }
  szBuf: array [0..255] of Char;

implementation

uses ExeParam, CallBack;

{$R *.DFM}

procedure TMyForm.CreateParams( var Params: TCreateParams );
  var
    P: TPoint;
  begin
    inherited CreateParams( Params );
    with Params do begin
      begin
        Style := Style or ws_Overlapped;
        { You can set your Application's Main Form to overlap
          any externally created window by setting WndParent
          to any valid window Handle. However, when your Main
          Form minimizes you may need to do something with
          TApplication.Icon since Delphi uses this Icon instead
          of your Main Form's Icon for minimized applications. }
        WndParent := Application.Handle;
        end;
      end;
    end;

procedure TMyForm.WndProc(var Msg: TMessage);
  var
    Handled: boolean;
    NumChars: Cardinal;
    pNewParams: Pointer;
  begin
    Handled := False;
    NumChars := 0;
    { Is this a custom message ? }
    if Msg.Msg >= $C000 then begin
      { Is this message carrying New Parameters ? }
      if (Msg.Msg = hParamMsg) and (Msg.lParam > 0) then
        with Msg do begin
        GetMem(pNewParams, lParam + 1);
        if pNewParams <> nil then
          NumChars := GlobalGetAtomName(wParam, pNewParams, lParam + 1);
        { Allow additional instances of this application to continue
          without receiving Focus. Parameter passed is ignored. }
        ReplyMessage(Ord(True));
        if NumChars > 0 then ProcessParams(PChar(pNewParams));
        FreeMem(pNewParams, lParam + 1);
        { Don't allow default WndProc to see this message }
        Handled := True;
        end;
      { Is this message from an additional instance of this
        application ? }
      if Msg.Msg = hShowWnd then begin
        { Main Form doesn't create an Icon, Application.Handle does }
        if IsIconic(Application.Handle) then begin
          Application.Restore;
          ShowWindow(Application.MainForm.Handle, SW_RESTORE)
          end
        else begin
          {$ifdef WIN32}
          SetForeGroundWindow(Application.MainForm.Handle);
          {$else}
          { SetWindowPos will bring Main Form to the top
            without sending additional messages to this
            WndProc. Using any API or Delphi function
            that sends additional messages to this WndProc
            will cause Delphi RunTime errors overflowing your
            application's CallStack. }
          SetWindowPos(Application.MainForm.Handle,
                       HWND_TOP,
                       Application.MainForm.Left,
                       Application.MainForm.Top,
                       Application.MainForm.Width,
                       Application.MainForm.Height,
                       SWP_SHOWWINDOW);
          {$endif}
          end;
        Handled := True;
        end;
      end;
    if not Handled then inherited WndProc(Msg);
    end;

procedure TMyForm.FormCreate(Sender: TObject);
begin
  { Caption must be set at run time. If your Delphi 2.0 design
    interface has a MainForm.Caption property (Object Inspector)
    identical to this run time pCaption then Run1Time will not
    behave properly. Only the length of pCaption will be examined
    when searching for previous instances of your MainForm.
    You may have additional text appended to pCaption. }

  Caption := StrPas(pCaption) + ' - example';

  { ReCreateWnd isn't needed for Run1Time. This is an additional
    example of Delphi's power and flexibility. ReCreate Main Form
    if you're setting any new TParam properties at Run time. You
    can call ReCreateWnd any time during application execution.
    ReCreateWnd will call CreateParams. }

  ReCreateWnd;
end;

end.
 
