unit Splash;                               {QUESTION: HOW DO I HIDE TO CAPTION?}

{THE POINT: To implement a splash screen component shared by my delphi apps.}

{THE GOAL: To bring up the splash screen AS SOON AS POSSIBLE after the exe is in control
and to be able to trigger that screen from inside the application as well.}

{THE IDEAS contained in here were developed from a message by Robert Saleas in which he
showed the Init/Deactivate idea and the component was made possible by Pat Ritchey pointing
out GetModuleFileName. The driving force however was to create a mechanism do the least
amount of work in the future. smile. that attitude runs through all my stuff.}

{WHY PUBLISHED: To bring up a splash screen is something everybody want to do but there's
no special delphi support to make it a no-brainer to have one. I'm not saying that delphi
should be any different, just that before this unit you had to kludge something.}

{HOW TO USE THE SPLASHSCREEN RIGHT NOW:
  Make 'Splash,' the first line in your project's uses list. that's all}

{HOW TO USE THE SPLASHSCREEN AS A COMPONENT:
  Register 'TSplashScreen'
  Place the component on a form
  Activate with 'SplashScreen.Execute'}

{HOW TO USE THE SPLASHSCREEN ON THE FLY
  Add 'Splash,' to any 'Uses' in the editor
  add a button, double click on it and type 'WITH TSPLASHSCREEN.CREATE(SELF) DO FREE;}


interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Rzlabel;


type
  TSplashScreenForm = class(TForm)
    Panel1: TPanel;
    Image1: TImage;
    Bevel1: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormDeactivate(Sender: TObject);
  end;

 TSplashScreen = class(TComponent)
 protected
   procedure SetTest(Value:Boolean);
   function GetTest:Boolean;
 public
   constructor Create(aOwner:TComponent);
   procedure Execute;
 published
   property Test:Boolean read GetTest write SetTest stored False;
   end;

implementation

{$R *.DFM}

{ Notice other units cannot access the form. }
var
  SplashScreen:  TSplashScreenForm;

{ This will make the form disappear, and get freed, when ANY other form or window pops up.}

procedure TSplashScreenForm.FormDeactivate(Sender: TObject);
begin
  SplashScreen.Free;
end;

constructor TSplashScreen.Create(aOwner:TComponent);
begin
  inherited create(aOwner);
  if not (csDesigning in ComponentState) then  {I'm not positive about this bit just yet.}
    Execute;
end;

procedure TSplashScreen.Execute;
begin
  SplashScreen:= TSplashScreenForm.Create(nil);
  SplashScreen.OnDeActivate:=SplashScreen.FormDeactivate;
  SplashScreen.Show;
  SplashScreen.Update;
end;

procedure TSplashScreen.SetTest(Value:Boolean);
begin
  Execute;
end;

function TSplashScreen.GetTest:Boolean;
begin
  Result:= Assigned(SplashScreen);
end;

procedure Initialize;
const
  BufSize=65; {length of buffer to hold 'modulefilename', e.g. path+file+dot+ext+null}
var
  Length: Integer;
  Pointer: PChar; {pointer to 'c' style string area}
begin
  getmem(Pointer,BufSize);
  Length:=GetModuleFileName(hInstance,Pointer,BufSize-1);
  if Length<BufSize-1 then {gets clipped if >BufSize, so only use if string unclipped,}
    if StrPos(Pointer,'.DCL')=nil then {no library extension in unclipped string}
      with TSplashScreen.Create(nil) do Free; {go ahead, show it!}
  freemem(Pointer,BufSize);
end;


initialization
  Initialize; {Automatically show the SplashScreen when this unit is initialized!}
end.
