{$I SRCOMOPT.INC}
unit Srmain;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, WMacro,
  StdCtrls, SysUtils, Messages, WMTypes, Dialogs;

type
  TSuperRecorder = class(TForm)
    MacroSet1: TMacroSet;
    Keys: TListBox;
    Status: TLabel;
    Assign: TButton;
    FastPlay: TCheckBox;
    SysWide: TCheckBox;
    GroupBox1: TGroupBox;
    IgnoreMouse: TRadioButton;
    MouseClicks: TRadioButton;
    AllMouse: TRadioButton;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    MaxMacs: TEdit;
    MaxLen: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    MacroName: TEdit;
    Label1: TLabel;
    Reinstall: TButton;
    SaveBtn: TButton;
    LoadBtn: TButton;
    Merge: TCheckBox;
    Label7: TLabel;
    RecordKeyMsg: TLabel;
    ScrapMacroKeyMsg: TLabel;
    CancelKeyMsg: TLabel;
    AssignShowMe: TButton;
    Label5: TLabel;
    MacroKeyComboBox1: TMacroKeyComboBox;
    CancelCheck: TCheckBox;
    Label8: TLabel;
    Memo1: TMemo;
    PauseKeyMsg: TLabel;
    DeleteBtn: TButton;
    GroupBox3: TGroupBox;
    MousePointer: TRadioButton;
    MouseWindow: TRadioButton;
    MouseScreen: TRadioButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure FastPlayClick(Sender: TObject);
    procedure SysWideClick(Sender: TObject);
    procedure IgnoreMouseClick(Sender: TObject);
    procedure MouseClicksClick(Sender: TObject);
    procedure AllMouseClick(Sender: TObject);
    procedure AssignClick(Sender: TObject);
    procedure ReinstallClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MacroSet1Assigned(Sender: TObject; Macro: pTMacroRec);
    procedure MacroSet1Recorded(Sender: TObject; State: TUserMacroState);
    procedure MacroSet1Played(Sender: TObject; State: TUserMacroState);
    procedure SaveBtnClick(Sender: TObject);
    procedure LoadBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AssignShowMeClick(Sender: TObject);
    procedure CancelCheckClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure MousePointerClick(Sender: TObject);
    procedure MouseWindowClick(Sender: TObject);
    procedure MouseScreenClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SuperRecorder: TSuperRecorder;
  OurWindow : HWND;
  OldCursor,BIGPOINT : HCursor;

implementation
const
  OffOn : array[boolean] of string = ('OFF','ON ');
  SlowMouseVDelta = 5;
  SlowMouseHDelta = 5;
  FastMouseVDelta = 30;
  FastMouseHDelta = 30;

{$R *.DFM}
{$R BIGPOINT.RES}

procedure TSuperRecorder.FastPlayClick(Sender: TObject);
begin
  MacroSet1.FastPlayback:=FastPlay.Checked;
end;

procedure TSuperRecorder.SysWideClick(Sender: TObject);
begin
  MacroSet1.SystemWide:=SysWide.Checked;
end;

procedure TSuperRecorder.IgnoreMouseClick(Sender: TObject);
begin
  If IgnoreMouse.Checked then MacroSet1.RecordMouse:=RMIgnore;
end;

procedure TSuperRecorder.MouseClicksClick(Sender: TObject);
begin
  If MouseClicks.Checked then MacroSet1.RecordMouse:=RMClicksAndDrags;
end;

procedure TSuperRecorder.AllMouseClick(Sender: TObject);
begin
  If AllMouse.Checked then MacroSet1.RecordMouse:=RMAll;
end;

procedure TSuperRecorder.AssignClick(Sender: TObject);
begin
  try
    With MacroSet1, MacroKeyComboBox1 do
      If (MKey>0) then Add(MKey,ScrapMacro,MacroName.Text,nil);
  except
  on Exception Do MessageBox(0,'An error occurred while adding the key macro','',MB_APPLMODAL);
  end;
end;

procedure TSuperRecorder.ReinstallClick(Sender: TObject);
begin
  Keys.Items.Clear;
  With MacroSet1 do begin
    If MacrosOn then MacrosOff;
    MaxMacros:=StrToInt(MaxMacs.Text);
    MaxMacroLen:=StrToInt(MaxLen.Text);
    MacrosOn;
  end;
end;

function GetTopLevelWindow(Handle : HWND) : HWND;
begin
  while (GetParent(Handle)<>0) do Handle:=GetParent(Handle);
  result:=Handle;
end;

procedure ShowMe; export;
begin
  PostMessage(OurWindow, WM_SYSCOMMAND, SC_HOTKEY, OurWindow);
  PostMessage(OurWindow, WM_SYSCOMMAND, SC_RESTORE, OurWindow);
end;

Procedure RunProg(const Prog : Pchar);
begin
  WinExec(Prog,SW_SHOWNORMAL);
end;

procedure RunCalc; export;
begin
  RunProg('CALC.EXE');
end;

procedure RunNotePad; export;
begin
  RunProg('NOTEPAD.EXE');
end;

procedure RunCalendar; export;
begin
  RunProg('CALENDAR.EXE');
end;

procedure RunClock; export;
begin
  RunProg('CLOCK.EXE');
end;

procedure MoveMouseUp; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.y:=MousePos.y-SlowMouseVDelta;
  If (MousePos.y<0) then MousePos.y:=0;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseLeft; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.x:=MousePos.x-SlowMouseHDelta;
  If (MousePos.x<0) then MousePos.x:=0;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseRight; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.x:=MousePos.x+SlowMouseHDelta;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseDown; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.y:=MousePos.y+SlowMouseVDelta;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseUpFast; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.y:=MousePos.y-FastMouseVDelta;
  If (MousePos.y<0) then MousePos.y:=0;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseLeftFast; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.x:=MousePos.x-FastMouseHDelta;
  If (MousePos.x<0) then MousePos.x:=0;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseRightFast; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.x:=MousePos.x+FastMouseHDelta;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseDownFast; export;
var
  MousePos : TPoint;
begin
  GetCursorPos(MousePos);
  MousePos.y:=MousePos.y+FastMouseVDelta;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseUpperLeft; export;
var
  MousePos : TPoint;
begin
  With MousePos do begin
    x:=0;
    y:=0;
  end;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseUpperRight; export;
var
  MousePos : TPoint;
begin
  With MousePos do begin
    x:=2048;
    y:=0;
  end;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseLowerLeft; export;
var
  MousePos : TPoint;
  ScreenHeight : Integer;
begin
  ScreenHeight := GetSystemMetrics(SM_CYSCREEN);
  With MousePos do begin
    x:=0;
    y:=ScreenHeight;
  end;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseLowerRight; export;
var
  MousePos : TPoint;
  ScreenWidth, ScreenHeight : Integer;
begin
  ScreenWidth := GetSystemMetrics(SM_CXSCREEN);
  ScreenHeight := GetSystemMetrics(SM_CYSCREEN);
  With MousePos do begin
    x:=ScreenWidth;
    y:=ScreenHeight;
  end;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MoveMouseCenter; export;
var
  MousePos : TPoint;
  ScreenWidth, ScreenHeight : Integer;
begin
  ScreenWidth := GetSystemMetrics(SM_CXSCREEN);
  ScreenHeight := GetSystemMetrics(SM_CYSCREEN);
  With MousePos do begin
    x:=ScreenWidth div 2;
    y:=ScreenHeight div 2;
  end;
  SetCursorPos(MousePos.x,MousePos.y);
end;

procedure MagnifyPointer; export;
var
  CurCursor : HCursor;
begin
  CurCursor:=GetCursor;
  If (CurCursor<>BIGPOINT) then begin
    OldCursor:=CurCursor;
    SetCursor(BIGPOINT);
  end;
end;

procedure ShrinkPointer; export;
var
  CurCursor : HCursor;
begin
  CurCursor:=GetCursor;
  If (CurCursor=BIGPOINT) then
    SetCursor(OldCursor);
end;

procedure TypeIt; export;
begin
  SuperRecorder.MacroSet1.TypeKeys('Written by Ken Henderson');
end;

procedure ClickLeftButton; export;
var
  MousePos : TPoint;
  Lparam : Longint;
  CurWin : HWND;
  CurRect : TRect;
begin

  GetCursorPos(MousePos);
  CurWin := WindowFromPoint(MousePos);
  GetWindowRect(CurWin, CurRect);
  Lparam:=MakeLong(MousePos.X-CurRect.Left, MousePos.Y-CurRect.Top);
  SendMessage(CurWin,WM_LBUTTONDOWN,0,Lparam);
  SendMessage(CurWin,WM_LBUTTONUP,0,Lparam);
end;

procedure DoubleClickLeftButton; export;
var
  MousePos : TPoint;
  Lparam : Longint;
  CurWin : HWND;
  CurRect : TRect;
begin
  ClickLeftButton;
  GetCursorPos(MousePos);
  CurWin := WindowFromPoint(MousePos);
  GetWindowRect(CurWin, CurRect);
  Lparam:=MakeLong(MousePos.X-CurRect.Left, MousePos.Y-CurRect.Top);
  SendMessage(CurWin,WM_LBUTTONDBLCLK,0,Lparam);
end;

procedure TSuperRecorder.FormCreate(Sender: TObject);
begin
  BIGPOINT:=LoadCursor(hInstance,pchar(255));
  With MacroSet1 do begin
    FastPlay.Checked:=FastPlayback;
    SysWide.Checked:=SystemWide;
    IgnoreMouse.Checked:=(RecordMouse=RMIgnore);
    MouseClicks.Checked:=(RecordMouse=RMClicksAndDrags);
    MousePointer.Checked:=(MouseBase=MBPointer);
    MouseWindow.Checked:=(MouseBase=MBWindow);
    MouseScreen.Checked:=(MouseBase=MBScreen);
    AllMouse.Checked:=(RecordMouse=RMAll);
    MaxMacs.Text:=IntToStr(MaxMacros);
    MaxLen.Text:=IntToStr(MaxMacroLen);
    CancelCheck.Checked:=CancelChecking;
    CancelKey:=VK_CANCEL;
    MacrosOn;
    RecordKeyMsg.Caption:=RecordKeyMsg.Caption+KeyNames[ToggleRecordKey];
    ScrapMacroKeyMsg.Caption:=ScrapMacroKeyMsg.Caption+KeyNames[Keys[-1]];
    CancelKeyMsg.Caption:=CancelKeyMsg.Caption+KeyNames[CancelKey];
    PauseKeyMsg.Caption:=PauseKeyMsg.Caption+KeyNames[PauseKey];
    Caption:='SuperRecorder Macro Recorder.  DLL Version: '+Version+'. Written by Ken Henderson.';
    If (ParamCount=0) then begin
    {Install some demo code macros}
    Add(MK_CTRLALTC,ScrapMacro,'Run CALC.EXE',RunCalc);
    Add(MK_CTRLALTN,ScrapMacro,'Run NOTEPAD.EXE',RunNotePad);
    Add(MK_CTRLSHIFTRBTN,ScrapMacro,'Run CALENDAR.EXE',RunCalendar);
    Add(MK_CTRLNUMSLASH,ScrapMacro,'Run CLOCK.EXE',RunClock);
    {$IFDEF BLINDGUY}
    Add(MK_SHIFTUP,ScrapMacro,'Move mouse up',MoveMouseUp);
    Add(MK_SHIFTDOWN,ScrapMacro,'Move mouse down',MoveMouseDown);
    Add(MK_SHIFTLEFT,ScrapMacro,'Move mouse left',MoveMouseLeft);
    Add(MK_SHIFTRIGHT,ScrapMacro,'Move mouse right',MoveMouseRight);
    Add(MK_SHIFTNUM8,ScrapMacro,'Move mouse up',MoveMouseUp);
    Add(MK_SHIFTNUM2,ScrapMacro,'Move mouse down',MoveMouseDown);
    Add(MK_SHIFTNUM4,ScrapMacro,'Move mouse left',MoveMouseLeft);
    Add(MK_SHIFTNUM6,ScrapMacro,'Move mouse right',MoveMouseRight);
    Add(MK_SHIFTHOME,ScrapMacro,'Move mouse to upper left',MoveMouseUpperLeft);
    Add(MK_SHIFTPGUP,ScrapMacro,'Move mouse to upper right',MoveMouseUpperRight);
    Add(MK_SHIFTEND,ScrapMacro,'Move mouse to lower left',MoveMouseLowerLeft);
    Add(MK_SHIFTPGDN,ScrapMacro,'Move mouse to lower right',MoveMouseLowerRight);
(*    Add(MK_CTRLSHIFTNUM7,ScrapMacro,'Move mouse to upper left',MoveMouseUpperLeft);
    Add(MK_CTRLSHIFTNUM9,ScrapMacro,'Move mouse to upper right',MoveMouseUpperRight);
    Add(MK_CTRLSHIFTNUM1,ScrapMacro,'Move mouse to lower left',MoveMouseLowerLeft);
    Add(MK_CTRLSHIFTNUM3,ScrapMacro,'Move mouse to lower right',MoveMouseLowerRight);
    Add(MK_CTRLSHIFTNUM5,ScrapMacro,'Move mouse to screen center',MoveMouseCenter);
*)
    Add(MK_CTRLSHIFTCENTER,ScrapMacro,'Move mouse to screen center',MoveMouseCenter);
    Add(MK_ALTSHIFTENTER,ScrapMacro,'Left mouse click',ClickLeftButton);
    Add(MK_CTRLALTENTER,ScrapMacro,'Left mouse double click', DoubleClickLeftButton);
    Add(MK_CTRLSHIFTM,ScrapMacro,'Magnify pointer',MagnifyPointer);
    Add(MK_CTRLSHIFTS,ScrapMacro,'Shrink pointer',ShrinkPointer);
(*    Add(MK_ALTSHIFTUP,ScrapMacro,'Move mouse quickly up',MoveMouseUpFast);
    Add(MK_ALTSHIFTDOWN,ScrapMacro,'Move mouse quickly down',MoveMouseDownFast);
    Add(MK_ALTSHIFTLEFT,ScrapMacro,'Move mouse quickly left',MoveMouseLeftFast);
    Add(MK_ALTSHIFTRIGHT,ScrapMacro,'Move mouse quickly right',MoveMouseRightFast);
    Add(MK_ALTSHIFTNUM8,ScrapMacro,'Move mouse quickly up',MoveMouseUpFast);
    Add(MK_ALTSHIFTNUM2,ScrapMacro,'Move mouse quickly down',MoveMouseDownFast);
    Add(MK_ALTSHIFTNUM4,ScrapMacro,'Move mouse quickly left',MoveMouseLeftFast);
    Add(MK_ALTSHIFTNUM6,ScrapMacro,'Move mouse quickly right',MoveMouseRightFast);
*)
    {$ENDIF}
    Add(MK_BACKQUOTE,ScrapMacro,'Type text',TypeIt);
    end else begin
       OpenDialog1.FileName:=UpperCase(ParamStr(1));
       ReadFromFile(OpenDialog1.FileName,true);
    end;
  end;
end;

procedure TSuperRecorder.MacroSet1Assigned(Sender: TObject; Macro: pTMacroRec);
var
  MMacs : word;
  MCnt : byte;
begin
  Keys.Items.Clear;
  MMacs:=MacroSet1.MaxMacros;
  For MCnt:=0 to pred(MMacs) do begin
    try
      Keys.Items.Add(MacroSet1.KeyNames[MacroSet1.Keys[MCnt]]+' -> '+MacroSet1.Names[MacroSet1.Keys[MCnt]]);
    except
      on EMacroNotFound Do {We don't care about not finding our macros};
    end;
  end;
end;

procedure TSuperRecorder.MacroSet1Recorded(Sender: TObject; State: TUserMacroState);
begin
  Status.Caption:=MacroSet1.StateName[State];
end;

procedure TSuperRecorder.MacroSet1Played(Sender: TObject; State: TUserMacroState);
begin
  Status.Caption:=MacroSet1.StateName[State];
end;

procedure TSuperRecorder.SaveBtnClick(Sender: TObject);
begin
  If (SaveDialog1.FileName='') and (OpenDialog1.FileName<>'') then
    SaveDialog1.FileName:=OpenDialog1.FileName;
  If (SaveDialog1.Execute) then MacroSet1.SaveToFile(SaveDialog1.FileName);
end;

procedure TSuperRecorder.LoadBtnClick(Sender: TObject);
begin
  If (OpenDialog1.Execute) then MacroSet1.ReadFromFile(OpenDialog1.FileName,Merge.Checked);
end;

procedure TSuperRecorder.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MacroSet1.MacrosOff;
  DestroyCursor(BIGPOINT);
end;

procedure TSuperRecorder.AssignShowMeClick(Sender: TObject);
Var
  MKey : Word;
begin
  try
    OurWindow:=SuperRecorder.Handle;
    With MacroSet1, MacroKeyComboBox1 do
      If (MKey>0) then Add(MKey,ScrapMacro,MacroName.Text,ShowMe);

  except
  on Exception Do MessageBox(0,'An error occurred while adding the Code Macro','',MB_APPLMODAL);
  end;
end;

procedure TSuperRecorder.CancelCheckClick(Sender: TObject);
begin
  MacroSet1.CancelChecking:=CancelCheck.Checked;
end;

procedure TSuperRecorder.DeleteBtnClick(Sender: TObject);
begin
  if (keys.itemindex <> -1) then
    macroset1.delete(macroset1.keys[succ(keys.itemindex)]);
end;

procedure TSuperRecorder.MousePointerClick(Sender: TObject);
begin
  If MousePointer.Checked then MacroSet1.MouseBase:=MBPointer;
end;

procedure TSuperRecorder.MouseWindowClick(Sender: TObject);
begin
  If MouseWindow.Checked then MacroSet1.MouseBase:=MBWindow;
end;

procedure TSuperRecorder.MouseScreenClick(Sender: TObject);
begin
  If MouseScreen.Checked then MacroSet1.MouseBase:=MBScreen;
end;

initialization
end.
