{$I WINMACRO.INC}
{WMImport.pas

This unit functions as the import facility for WINMACRO.DLL, a keyboard/mouse/code
macro engine for Windows

Written by Ken Henderson.

Copyright (c) 1994 Ken Henderson.  All Rights Reserved.

Last modification: 11-23-94


I've used const parameters very liberally here, even in situations where it would have
required less memory to have used non-const parameters.  The reasons for this are
manifold.  First, I believe it's better form to use const for parameters that cannot
be changed.  Second, someday a word variable in BP might be the same size as a pointer
variable, making the trade-off a wash, with the scales tipping toward the use of const
since it also prevents changes to parameters the developer believes should not need
to be changed.  For now, a const parameter whose type is smaller than four bytes
wastes a small amount of memory.  However, I think it's poor form to write code which
is intrinsically dependent on variable sizes never changing, thus my liberal use of
const parameters.
}
unit WMimport;

interface

Uses WinTypes,
     WMTypes;

function LoadMsg(MsgNum: Word): string;
function GetKeyCtrl(const MKey : word) : boolean;
procedure SetKeyCtrl(var MKey : Word; const Ctrl : boolean);
function GetKeyAlt(const MKey : word) : boolean;
procedure SetKeyAlt(var MKey : Word; const Alt : boolean);
function GetKeyShift(const MKey : word) : boolean;
procedure SetKeyShift(var MKey : Word; const Shift : boolean);
procedure GetShiftStates(var MKey : Word);
function GetPossibleMKeyIndex(const MKey : Word) : Integer;
function GetPossibleMKey(const Index : Word) : Word;
function GetMaxPossibleMKeys : Word;
function GetMacroByKey(const MH : pTMacroContext; const MKey : word) : pTMacroRec;
function GetMacroByIndex(const MH : pTMacroContext; const Index : Word) : pTMacroRec;
function GetMacroKey(const pInMacro : pTMacroRec) : Word;
procedure SetMacroKey(var pInMacro : pTMacroRec; const MKey : Word);
function GetMacroName(const Macro : pTMacroRec): Pchar;
procedure SetMacroName(var pInMacro : pTMacroRec; const Name : PChar);
function GetMacroCodeHook(const pInMacro : pTMacroRec) : CodeHookProc;
procedure SetMacroCodeHook(var pInMacro : pTMacroRec; const CodeHook : CodeHookProc);
function GetKeyName(const MKey : Word; const MKeyText : PChar; const MaxTextLen : word) : PChar;
function GetToggleRecordKey : Word;
procedure SetToggleRecordKey(const MKey : Word);
function GetCancelKey : Word;
function SetCancelKey(const MKey : Word) : boolean;
function GetPauseKey : Word;
function SetPauseKey(const MKey : Word) : boolean;
function WriteMacros(const MH : pTMacroContext; const FileName : PChar) : byte;
function ReadMacros(const MH : pTMacroContext; const FileName : PChar; const Merge : boolean) : byte;
procedure ToggleRecording;
procedure RecordingOn;
procedure RecordingOff;
function GetMacroState : TMacroState;
function InstallMacro(const MH : pTMacroContext;
		       const MKey : word;
                      const Macro : pTMacroRec;
                      const Name : Pchar;
                      const CodeHook : CodeHookProc) : byte;
procedure PlaybackMacro(const MacroToPlay : ptMacroRec);
procedure TypeKeys(const KeyChars : PChar);
function GetScrapMacro : pTMacroRec;
procedure PlaybackScrap;
function InstallHook(const _CallingWindowHook : HWND; Const _MaxMacroNum, _MaxMacroLen : word;
                     const _FastPlayback, SystemWide : boolean;
                     const _RecordMouse : TRecordMouse;
                     const _MouseBase : TMouseBase;
                     const _CancelChecking : boolean) : pTMacroContext;
procedure UninstallHook(var MH : pTMacroContext);
function GetWinmacroVersion : Word;

implementation

function LoadMsg; external 'WINMACRO';
function GetKeyCtrl; external 'WINMACRO';
procedure SetKeyCtrl; external 'WINMACRO';
function GetKeyAlt; external 'WINMACRO';
procedure SetKeyAlt; external 'WINMACRO';
function GetKeyShift; external 'WINMACRO';
procedure SetKeyShift; external 'WINMACRO';
procedure GetShiftStates; external 'WINMACRO';
function GetPossibleMKeyIndex; external 'WINMACRO';
function GetPossibleMKey; external 'WINMACRO';
function GetMaxPossibleMKeys; external 'WINMACRO';
function GetMacroByKey; external 'WINMACRO';
function GetMacroByIndex; external 'WINMACRO';
function GetMacroKey; external 'WINMACRO';
procedure SetMacroKey; external 'WINMACRO';
function GetMacroName; external 'WINMACRO';
procedure SetMacroName; external 'WINMACRO';
function GetMacroCodeHook; external 'WINMACRO';
procedure SetMacroCodeHook; external 'WINMACRO';
function GetToggleRecordKey; external 'WINMACRO';
procedure SetToggleRecordKey; external 'WINMACRO';
function GetCancelKey; external 'WINMACRO';
function SetCancelKey; external 'WINMACRO';
function GetPauseKey; external 'WINMACRO';
function SetPauseKey; external 'WINMACRO';
function GetKeyName; external 'WINMACRO';
function WriteMacros; external 'WINMACRO';
function ReadMacros; external 'WINMACRO';
procedure ToggleRecording; external 'WINMACRO';
procedure RecordingOn; external 'WINMACRO';
procedure RecordingOff; external 'WINMACRO';
function GetMacroState; external 'WINMACRO';
function InstallMacro; external 'WINMACRO';
procedure PlaybackMacro; external 'WINMACRO';
procedure TypeKeys; external 'WINMACRO';
function GetScrapMacro; external 'WINMACRO';
procedure PlaybackScrap; external 'WINMACRO';
function InstallHook; external 'WINMACRO';
procedure UninstallHook; external 'WINMACRO';
function GetWinmacroVersion; external 'WINMACRO';

end.
