#ifndef __WMTYPES_H
#define __WMTYPES_H
#endif

#ifndef BYTE
typedef unsigned char BYTE;
#endif
#ifndef BOOLEAN
typedef unsigned char BOOLEAN;
#endif
#ifndef LPUINT
typedef UINT FAR * LPUINT;
#endif


#define SKCtrl 			0
#define SKAlt  			1
#define SKShift			2
#define SKCapslock		3
#define SKNumlock			4
#define SKScrollLock		5

#define MaxMacroContext 9

/*Messages used by the system */

#define  MM_MACROFIRST WM_USER+500
#define  MM_STARTRECORD MM_MACROFIRST
#define  MM_STOPRECORD MM_STARTRECORD + 1

/*  I consider this method -- that of adding just 1 to each successive
	message, with each message dependent on the previous one for its
	index -- preferable because adding or deleting a message only requires
	the changing of the one immediately following the message, not all
	of the following messages.

*/

#define MM_STARTPAUSE MM_STOPRECORD + 1
#define MM_STOPPAUSE MM_STARTPAUSE + 1
#define MM_STARTPLAYBACK MM_STOPPAUSE + 1
#define MM_STOPPLAYBACK  MM_STARTPLAYBACK + 1
#define MM_PAUSEPLAYBACK  MM_STOPPLAYBACK + 1   /*Necessitated by SysModal dialogs*/
#define MM_RESUMEPLAYBACK  MM_PAUSEPLAYBACK + 1 /*Companion to MM_PAUSEPLAYBACK */

/* Messages sent to the calling task */

#define MM_RECORDON  MM_RESUMEPLAYBACK + 1
#define MM_RECORDOFF  MM_RECORDON + 1
#define MM_PLAYBACKON  MM_RECORDOFF + 1
#define MM_PLAYBACKOFF  MM_PLAYBACKON + 1
#define MM_PLAYBACKPAUSED  MM_PLAYBACKOFF + 1
#define MM_PLAYBACKRESUMED  MM_PLAYBACKPAUSED + 1
#define MM_RECORDCHANGE  MM_PLAYBACKRESUMED + 1
#define MM_PLAYBACKCHANGE  MM_RECORDCHANGE + 1
#define MM_MACROASSIGNED  MM_PLAYBACKCHANGE + 1
#define MM_MACROLAST  MM_MACROASSIGNED

#define MACRONAMESTRINGLEN 60

#define CtrlOn 			0x01
#define AltOn   			0x02
#define ShiftOn  			0x04
#define CapslockOn 		0x08
#define NumlockOn  		0x10
#define ScrollLockOn 	0x20

  /*The following (stored as part of Toggles in TMHDR) prevents the ScrapMacro
  from being accidentally deallocated or used in other inappropriate situations.
  It accomplishes this without the need for yet another flag variable
  */

#define ScrapMacroStatusBit 0x80

  /*This array sets up the flag representative of each shift key
  */

BYTE ShiftStateOn[6] = {CtrlOn, AltOn, ShiftOn, CapslockOn, NumlockOn, ScrollLockOn};


  /*The following is used by the macro file read/write routines
  */

#define UnKnownIOError 0xFF;;

/*Messages -- implemented as a Windows string table
*/

#ifndef __WMMSGS_H
#include <wmmsgs.h>
#endif

/*Return codes
*/

#define MRSuccess 0
#define MRNoMacroRoom 1

/*MRSevereError allows us to easily distinguish between severe and non-
  severe errors.  It serves as a marker dividing non-severe errors from
  severe ones.  Errors above MRSevereError are severe errors, those under
  it aren't.  The actual code returned may or may not be of interest
  to the programmer and he can decide whether to do something specific to
  a given return code or based solely on whether a severe error occurred.
*/

#define MRSevereError 0x80
#define MROutOfMemory 0x81


/*MKey constant definitions for all possible keys that the macro engine
  recognizes.  Constants follow the form of MK_XXXX where XXXXX is some
  combination of 'Ctrl', 'Alt' and 'Shift' and an unshifted key name, depending on
  the key.  For example, Control+Alt+Shift+F10 is setup as MK_CtrlAltShiftF10.
  These constants can be used for the MKey parameter when application installs
  new macros.
*/


#ifndef __WMKEYS_H
#include <wmkeys.h>
#endif

  /*Current state of the macro engine*/

#define MSNONE				0
#define MSRECORD			1
#define MSPLAYBACK		2
#define MSPAUSERECORD	3

  /*Used to tell the user the current recording/playback status*/

#define UMSNONE			0
#define RSOFF				1
#define RSON				2
#define RSPAUSED			3
#define RSRESUMED			4
#define PSOFF				5
#define PSON				6
#define PSPAUSED			7
#define PSRESUMED			8

  /*Used by the engine to determine what types of mouse events to record*/

#define RMIGNORE				0
#define RMCLICKSANDDRAGS	1
#define RMALL					2

/*Used to set the basis for mouse actions.  They can correspond to the screen
  as a whole, to the current window or to the initial position of the mouse pointer
*/


#define MBSCREEN				0
#define MBWINDOW				1
#define MBPOINTER				2

  /*TMHDR's form the first of two fields in TMacroRec's.  It might seem
  pointless to group these fields into their own record, then use the
  record in yet another record, but consider:  Memory is allocated
  to TMacroRec's based on the actual length of the macro (usually the scrap
  macro) being copied to the new macro.  When this happens, having the fields
  that never vary in size in a separate record struct allows the memory needed
  to be easily calculated using the following formula:

  MemNeeded:=sizeof(TMHDR)+(Length*sizeof(TEventMsg));

  where Length is the Length field from the source macro.  Sizeof(TEventMsg)
  is multiplied by Length because the Msgs field of a TMacroRec is an array
  of TEventMsg's.  It's this convention that allows the macros to vary in
  size without resorting to a linked list.  By grouping all but the variant
  field into a separate record structure, and then using that sub-record
  in all calculations for the "parent" record, we leave open the possibility
  that fields could be added or deleted from the TMacroRec (via the TMHDR,
  of course) without changing the calculation forumla.  The only other way
  to accomplish this flexibility would be to place "tag" variables in the
  record structure, between which all fields insertions or deletions would
  have to be made.  This solution is much more elegant.  The only negative
  to this solution is that the subrecord must proceed any reference to the fields
  that have been placed within it (of course), hence it's abbreviated name.*/

  typedef struct {
	 BYTE Toggles; /*Holds the initial status of Capslock, Numlock and ScrollLock*/
	 UINT MKey;   /*Macro hotkey*/
	 UINT MacroEnd; /*The index of the last message recorded in the macro*/
	 BYTE MouseBase; /*Stores the "home position" for mouse messages*/
	 void far * CodeHook; /*A routine to call rather than play back keys*/
	 LPSTR Name;            /*Macro name*/
	 } TMHDR;

  /*  Last 16 bytes reserved */
  #define MaxUINT 65520

  /*Even though memory is allocated for TMacroRec's based on the MaxMacroLen parameter
	passed into the InstallHook routine, type TMsgs is defined using the following
	constant so that the TEventMsgs that make up TMsgs can be viewed using the debugger.
	This is necessary because the debugger looks at the type definition to determine
	the max array index of array variables.  That is,

	TMsgs = array[1..1] of TEventMsg;

	would work just fine for our purposes since the memory TMsgs uses is allocated dynamically
	anyway.  However, this convention would fool the debugger, therefore we use the following.
	The same convention is used below with pTMacros.*/

#define TMsgMax ((MaxUINT-sizeof(TMHDR)) / sizeof(EVENTMSG));

typedef EVENTMSG TMsgs[1000];

  /*The actual definition of a macro record*/

typedef struct {
	 TMHDR MHDR;
	 TMsgs Msgs;
  } TMacroRec;

  /*I don't establish pointers to types that don't yet exist; I build a pointer to
	a type after I've established the type.  There is seldom a need for setting up the
	pointer first (a linked list would be one example), and doing it without a need
	for it has always seemed awkward to me.
	*/

  typedef TMacroRec far * pTMacroRec;

#define MaxMacs MaxUINT / sizeof(pTMacroRec)

  /*The array of macro handles stored in each macro context*/

typedef pTMacroRec TMacros[1000];
typedef TMacros far * pTMacros;

  /*A specialized TMessage used to send messages to the CallingWindow*/


typedef struct {
	 UINT Msg;
	 UINT State;     /*Used in MM_XXXXCHANGED messages*/
	 long Macro;  /*Used only in the MM_MACROASSIGNED message*/
	 long Result;
  }   TWMMacMsg;

 /*The macro context type.  It establishes a little macro "world", if you will,
  that is made the _current_ "world" by searching through the hotkeys of the
  allocated contexts for a match to the key or mouse button just pressed.  If a
  context can be located, the variables that were passed to it when InstallHook
  was called are copied to global versions of the same and the engine is allowed
  to proceed with the playback operation.
  */

typedef struct {
  HANDLE TaskHandle;
  pTMacros Macros;
  pTMacroRec CurrentMacroRec;
  /*The following are passed in from the caller*/
  HWND CallingWindowHook;
  UINT MaxMacroNum;
  UINT MaxMacroLen;
  BOOLEAN FastPlayback;
  HANDLE TaskScope;
  UINT LoMsgBound;
  UINT HiMsgBound;
  BYTE MouseBase;
  BOOLEAN CancelChecking;
} TMacroContext;

typedef TMacroContext far * pTMacroContext;

