unit PXPEditr;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, DsgnIntf, Menus, PXPictEd,
  DB, DBTables;

type
  TPictureEditDlg = class(TForm)
    PictLib: TComboBox;
    PictTest: TPXPictureEdit;
    PictDesc: TEdit;
    Pict: TEdit;
    Status: TPanel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    AutoFill: TCheckBox;
    UseBtn: TBitBtn;
    AddBtn: TBitBtn;
    DelBtn: TBitBtn;
    DescLbl: TLabel;
    TestLbl: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
    procedure AutoFillClick(Sender: TObject);
    procedure PictLibChange(Sender: TObject);
    procedure PictChange(Sender: TObject);
    procedure UseBtnClick(Sender: TObject);
    procedure DelBtnClick(Sender: TObject);
    procedure PictDescExit(Sender: TObject);
    procedure PictTestChange(Sender: TObject);
  private
    FPXTransfer: TPXPictureValidator;
    PictDescLib: TStringList;
    procedure SetTestStatus;
    procedure SetPictStatus;
    procedure SetPXTransfer(Value: TPXPictureValidator);
  public
    SaveDir: String;
    procedure LoadLibrary;
    property PXTransfer: TPXPictureValidator read FPXTransfer write SetPXTransfer;
  end;

  TPXPictureProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TPXPropertyEditor = class(TDefaultEditor)
  public
    procedure EditProperty(PropertyEditor: TPropertyEditor; var Continue, FreeEditor: Boolean); virtual;
  end;

var
  PictureEditDlg : TPictureEditDlg;

implementation

const
  DATA_FILE = 'TPXPICTR.DAT';
{$R *.DFM}

{
***********************************************************************
TPictureEditDlg
***********************************************************************
}
procedure TPictureEditDlg.FormCreate(Sender: TObject);
begin
PictDescLib := TStringList.Create;
SaveDir := '';
end;

procedure TPictureEditDlg.SetPXTransfer(Value: TPXPictureValidator);
begin
FPXTransfer := Value;
Pict.Text := Value.Picture;
AutoFill.Checked := Value.AutoFill;
end;

procedure TPictureEditDlg.LoadLibrary;
var
  tfs: TFileStream;
  tr: TReader;
  n,i:  LongInt;
  s: String;
begin
try
  tfs := TFileStream.Create(SaveDir + DATA_FILE,fmOpenRead);
  tr := TReader.Create(tfs,1024);
  try
    n := tr.ReadInteger;
    for i := 0 to n - 1 do
      begin
      s := tr.ReadString;
      PictLib.Items.Add(s);
      s := tr.ReadString;
      PictDescLib.Add(s);
      end;
  finally
    tr.Free;
    tfs.Free;
  end;
except
  on EFOpenError do;
  on E:Exception do Application.HandleException(E);
end;
end;

procedure TPictureEditDlg.SetTestStatus;
var
  s: string;
begin
  s := PictTest.Text;
  case PictTest.PXPicture.Eval(s,PictTest.PXPicture.AutoFill) of
    prComplete: Status.Caption := 'prComplete';
    prIncomplete: Status.Caption := 'prIncomplete';
    prEmpty: Status.Caption := 'prEmpty';
    prError: Status.Caption := 'prError';
    prSyntax: Status.Caption := 'prSyntax';
    prAmbiguous: Status.Caption := 'prAmbiguous';
    prIncompNoFill: Status.Caption := 'prIncompNoFill';
  end;
end;

procedure TPictureEditDlg.AddBtnClick(Sender: TObject);
var
  i: integer;
  dup: Boolean;
begin
  with PictLib do
    begin
    dup := False;
    i := 0;
    for i := 0 to Items.Count - 1 do
      if Items[i] = Pict.Text then
        begin
        dup := True;
        Exit;
        end;
      if not dup then
        begin
        Items.Add(Pict.Text);
        PictDescLib.Add(PictDesc.Text);
        end;
    end;
end;

procedure TPictureEditDlg.CancelBtnClick(Sender: TObject);
begin
Close;
end;

procedure TPictureEditDlg.OkBtnClick(Sender: TObject);
var
  tfs: TFileStream;
  tw: TWriter;
  n,i:  LongInt;
begin
try
  PXTransfer.Assign(PictTest.PXPicture);
  tfs := TFileStream.Create(SaveDir + DATA_FILE,fmCreate);
  tw := TWriter.Create(tfs,1024);
  try
    tw.WriteInteger(PictLib.Items.Count);
    for i := 0 to PictLib.Items.Count - 1 do
      begin
      tw.WriteString(PictLib.Items[i]);
      tw.WriteString(PictDescLib[i]);
      end;
  finally
    tw.Free;
    tfs.Free;
  end;
except
  on E:Exception do
    begin
    Application.HandleException(E);
    Close;
    end;
end;
Close;
end;

procedure TPictureEditDlg.AutoFillClick(Sender: TObject);
begin
PictTest.PXPicture.AutoFill := AutoFill.Checked;
end;

procedure TPictureEditDlg.PictLibChange(Sender: TObject);
begin
with PictLib do
  begin
  UseBtn.Enabled := True;
  DelBtn.Enabled := True;
  PictDesc.Enabled := True;
  DescLbl.Font.Color := clBlack;
  PictDesc.Text := PictDescLib[ItemIndex];
  end;
end;

procedure TPictureEditDlg.PictChange(Sender: TObject);
begin
SetPictStatus;
end;

procedure TPictureEditDlg.SetPictStatus;
begin
with Pict do
  begin
  try
    PictTest.PXPicture.Picture := Text;
    AddBtn.Enabled := (Text <> '') and (Text <> PictLib.Text);
    Status.Caption := 'Ok';
    OkBtn.Enabled := Text <> '';
    PictTest.Enabled := True;
    TestLbl.Font.Color := clBlack;
  except
    AddBtn.Enabled := False;
    Status.Caption := 'prSyntax';
    OkBtn.Enabled := AddBtn.Enabled;
    PictTest.Enabled := False;
    TestLbl.Font.Color := clGray;
  end;
  end;
end;

procedure TPictureEditDlg.UseBtnClick(Sender: TObject);
begin
Pict.Text := PictLib.Text;
end;

procedure TPictureEditDlg.DelBtnClick(Sender: TObject);
begin
with PictLib do
  begin
  PictDescLib.Delete(ItemIndex);
  Items.Delete(ItemIndex);
  ItemIndex := -1;
  end;
PictDesc.Text := '';
PictDesc.Enabled := False;
DescLbl.Font.Color := clGray;
DelBtn.Enabled := False;
UseBtn.Enabled := False;
end;

procedure TPictureEditDlg.PictDescExit(Sender: TObject);
begin
PictDescLib[PictLib.ItemIndex] := PictDesc.Text;
end;


procedure TPictureEditDlg.PictTestChange(Sender: TObject);
begin
SetTestStatus;
end;


{
***********************************************************************
TPXPictureProperty
***********************************************************************
}
procedure TPXPictureProperty.Edit;
var
  i: integer;
begin
  PictureEditDlg := TPictureEditDlg.Create(Application);
  try
    with PictureEditDlg do
      begin
      PXTransfer := TPXPictureValidator(GetOrdValue);
      SaveDir := PrivateDirectory + '\';
      LoadLibrary;
      ShowModal;
      end;
  finally
    PictureEditDlg.Free;
  end;
end;

function TPXPictureProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paSubProperties];
end;

{
***********************************************************************
TPXPictureEditor
***********************************************************************
}
procedure TPXPropertyEditor.EditProperty(PropertyEditor: TPropertyEditor; var Continue, FreeEditor: Boolean);
var
  PropName: string;
begin
  PropName := PropertyEditor.GetName;
  if (CompareText(PropName, 'PXPICTURE') = 0) then
    begin
    PropertyEditor.Edit;
    Continue := False;
    end;
end;

end.
