unit PXPictEd;

{$O+,F+,X+,I-,S-}
interface

uses
  StdCtrls, Forms, Controls, Graphics, Menus, Classes,
  SysUtils, DB, DBTables, Messages, WinTypes, WinProcs, Dsgnintf;

type
  {
  Possible status values for a TPXPictureValidater's picture
  All elements are the status of the input with the exception of prSyntax,
  which refers to the picture itself.
  }
  TPXPictStatus = (prComplete, prIncomplete, prEmpty, prError, prSyntax, prEmptyPicture, prAmbiguous, prIncompNoFill);

  {
  TPXPictureValidater raises the following exception to
  indicate an invalid picture.
  }
  EPXPictureInvalid = class(Exception);

  {
  TPXPictureValidater class

  The source for this class was extracted and modified
  from Borland's Turbo Pascal 7.0 runtime library.
  In general the modifications brought the code up to Delphi
  standards.
  }
  TPXPictureValidator = class(TPersistent)
  private
    Pic: PString;
    Options: Word;
    FPicture:  String;
    FAutoFill: Boolean;
    procedure SetPicture(Value: String);
    function IsComplete(Rslt: TPXPictStatus): Boolean;
    function IsIncomplete(Rslt: TPXPictStatus): Boolean;
  public
    constructor Create;
    destructor Destroy;

    function HasPicture: Boolean;
    procedure Assign(Value: TPXPictureValidator);
    function IsValid(const S: string): Boolean;
    function Eval(var Input: string; AutoFill: Boolean): TPXPictStatus;

  published
    property Picture: string read FPicture write SetPicture;
    property AutoFill: Boolean read FAutoFill write FAutoFill default True;
  end;

  {
  TPXCustomPictureEdit class
  }

  {OnError Event type handler}
  TPXPErrorEvent = procedure (Sender: TObject; ErrStr: String) of object;

  TPXCustomPictureEdit = class(TCustomEdit)
  private
    FPXPicture: TPXPictureValidator;
    FOnError: TPXPErrorEvent;

    function PrepareText(const NewChar: Char): string;
    function FillText(var s: string): TPXPictStatus;
    procedure SetPXPicture(Value: TPXPictureValidator);
  protected
    procedure KeyPress(var Ch: char); override;
    procedure Error(ErrStr: String); dynamic;
    function  EditCanModify: Boolean; virtual;

    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property PXPicture: TPXPictureValidator read FPXPicture write SetPXPicture;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property Text;
    property Visible;

    {Events}
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnError: TPXPErrorEvent read FOnError write FOnError;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

    public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    published
    property TabStop;

  end; {TPXCustomPictureEdit class}

  {
  TPXPictureEdit class
  }
  TPXPictureEdit = class(TPXCustomPictureEdit)
  published
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property PXPicture;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property Text;
    property Visible;

    {Events}
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnError;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;  {TPXPictureEdit class}

  {
  TPXDBPictureEdit class
  }
  TPXDBPictureEdit = class(TPXCustomPictureEdit)
  private
    FDataLink: TFieldDataLink;
    FCanvas: TControlCanvas;
    FAlignment: TAlignment;
    FFocused: Boolean;
    FTextMargin: Integer;
    procedure CalcTextMargin;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetFocused(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure Change; override;
    function EditCanModify: Boolean; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Reset;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
  published
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property PXPicture;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property TabOrder;
    property Text;
    property Visible;

    {Events}
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnError;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;  {TPXDBPictureEdit class}

procedure Register;

implementation

uses
  PXPEditr;

{
***********************************************************************
Component Registration
***********************************************************************
}
procedure Register;
begin
  RegisterComponents('Additional', [TPXPictureEdit]);
  RegisterComponents('Data Controls', [TPXDBPictureEdit]);
  RegisterComponentEditor(TPXDBPictureEdit, TPXPropertyEditor);
  RegisterComponentEditor(TPXPictureEdit, TPXPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TPXPictureValidator), Nil, '', TPXPictureProperty);
end;

{
***********************************************************************
TPXPictureValidator Methods
***********************************************************************
}

{
The following 3 functions: IsNumber, IsLetter, and IsSpecial are
helper funtions for the TPXPictureValidator Picture function.
They are not declared as class methods so that calls can be near.
}

{
function IsNumber
returns True if Chr is in ['0'..'9']
}
function IsNumber(Chr: Char): Boolean; near; assembler;
asm
        XOR     AL,AL
        MOV     Ch,Chr
        CMP     Ch,'0'
        JB      @@1
        CMP     Ch,'9'
        JA      @@1
        INC     AL
@@1:
end;

{
function IsLetter
returns True if Chr is in ['A'..'z']
}
function IsLetter(Chr: Char): Boolean; near; assembler;
asm
        XOR     AL,AL
        MOV     Cl,Chr
        AND     Cl,0DFH
        CMP     Cl,'A'
        JB      @@2
        CMP     Cl,'Z'
        JA      @@2
@@1:    INC     AL
@@2:
end;

{
function IsSpecial
returns True if Chr is a reserved Picture character
}
function IsSpecial(Chr: Char; const Special: string): Boolean; near; assembler;
asm
        XOR     AH,AH
        LES     DI,Special
        MOV     AL,ES:[DI]
        INC     DI
        MOV     CH,AH
        MOV     CL,AL
        MOV     AL,Chr
        REPNE   SCASB
        JCXZ    @@1
        INC     AH
@@1:    MOV     AL,AH
end;


{
constructor Create
}
constructor TPXPictureValidator.Create;
begin
  inherited Create;
  Pic := Nil;
  FAutoFill := True;
  Picture := '';
end;

{
destructor Destroy
}
destructor TPXPictureValidator.Destroy;
begin
DisposeStr(Pic);
inherited Destroy;
end;

{
procedure Assign
Assigns the properties of another TPXPictureValidator
}
procedure TPXPictureValidator.Assign(Value: TPXPictureValidator);
begin
if Assigned(Value) then
  begin
  SetPicture(Value.Picture);
  FAutoFill := Value.AutoFill;
  end;
end;

{
function HasPicture
Returns True if Picture is defined
}
function TPXPictureValidator.HasPicture: Boolean;
begin
HasPicture := Pic <> Nil;
end;

{
procedure SetPicture
Sets TPXPictureValidator's Picture property
}
procedure TPXPictureValidator.SetPicture(Value: String);
var
  OldPic:  PString;
  pr:      TPXPictStatus;
begin
if Value <> Picture then
  if Value = '' then
    {Remove Picture}
    begin
    FPicture := Value;
    DisposeStr(Pic);
    Pic := Nil;
    end
  else
    begin
    {Save old Picture in case new picture is invalid}
    OldPic := Pic;
    Pic := NewStr(Value);
    if IsValid('') then
      begin
      FPicture := Value;
      DisposeStr(OldPic);
      end
    else
      {New Picture is Invalid, so restore old picture}
      begin
      DisposeStr(Pic);
      Pic := OldPic;
      {raise EPXPictureInvalid}
      raise EPXPictureInvalid.Create(Value + ' is not a valid picture.');
      end;
    end; {Value <> ''}
end;


{
function IsValid
Returns True if input, s, conforms to picture specs
}
function TPXPictureValidator.IsValid(const S: string): Boolean;
var
  Str: String;
  Rslt: TPXPictStatus;
begin
  Str := S;
  if Pic = Nil then
    IsValid := True
  else
    begin
    Rslt := Eval(Str, False);
    IsValid := (Rslt = prComplete) or (Rslt = prEmpty);
    end;
end;


{
function IsComplete
Used by Eval
}
function TPXPictureValidator.IsComplete(Rslt: TPXPictStatus): Boolean;
begin
  IsComplete := Rslt in [prComplete, prAmbiguous];
end;

{
function IsIncomplete
Used by Eval
}
function TPXPictureValidator.IsIncomplete(Rslt: TPXPictStatus): Boolean;
begin
  IsIncomplete := Rslt in [prIncomplete, prIncompNoFill];
end;

{
function Eval
This is the picture engine
}
function TPXPictureValidator.Eval(var Input: string; AutoFill: Boolean): TPXPictStatus;
var
  I, J: Byte;
  Rslt: TPXPictStatus;
  Reprocess: Boolean;

  function Process(TermCh: Byte): TPXPictStatus;
  var
    Rslt: TPXPictStatus;
    Incomp: Boolean;
    OldI, OldJ, IncompJ, IncompI: Byte;

    { Consume input }
    procedure Consume(Ch: Char);
    begin
      Input[J] := Ch;
      Inc(J);
      Inc(I);
    end;

    { Skip a character or a picture group }
    procedure ToGroupEnd(var I: Byte);
    var
      BrkLevel, BrcLevel: Integer;
    begin
      BrkLevel := 0;
      BrcLevel := 0;
      repeat
        if I = TermCh then Exit;
        case Pic^[I] of
          '[': Inc(BrkLevel);
          ']': Dec(BrkLevel);
          '{': Inc(BrcLevel);
          '}': Dec(BrcLevel);
          ';': Inc(I);
          '*':
            begin
              Inc(I);
              while IsNumber(Pic^[I]) do Inc(I);
              ToGroupEnd(I);
              Continue;
            end;
        end;
        Inc(I);
      until (BrkLevel = 0) and (BrcLevel = 0);
    end;

    { Find the a comma separator }
    function SkipToComma: Boolean;
    begin
      repeat ToGroupEnd(I) until (I = TermCh) or (Pic^[I] = ',');
      if Pic^[I] = ',' then Inc(I);
      SkipToComma := I < TermCh;
    end;

    { Calclate the end of a group }
    function CalcTerm: Byte;
    var
      K: Byte;
    begin
      K := I;
      ToGroupEnd(K);
      CalcTerm := K;
    end;

    { The next group is repeated X times }
    function Iteration: TPXPictStatus;
    var
      Itr, K, L: Byte;
      Rslt: TPXPictStatus;
      NewTermCh: Byte;
    begin
      Itr := 0;
      Iteration := prError;

      Inc(I);  { Skip '*' }

      { Retrieve number }

      while IsNumber(Pic^[I]) do
      begin
        Itr := Itr * 10 + Byte(Pic^[I]) - Byte('0');
        Inc(I);
      end;

      if I > TermCh then
      begin
        Iteration := prSyntax;
        Exit;
      end;

      K := I;
      NewTermCh := CalcTerm;

      { If Itr is 0 allow any number, otherwise enforce the number }
      if Itr <> 0 then
      begin
        for L := 1 to Itr do
        begin
          I := K;
          Rslt := Process(NewTermCh);
          if not IsComplete(Rslt) then
          begin
            { Empty means incomplete since all are required }
            if Rslt = prEmpty then Rslt := prIncomplete;
            Iteration := Rslt;
            Exit;
          end;
        end;
      end
      else
      begin
        repeat
          I := K;
          Rslt := Process(NewTermCh);
        until not IsComplete(Rslt);
        if (Rslt = prEmpty) or (Rslt = prError) then
        begin
          Inc(I);
          Rslt := prAmbiguous;
        end;
      end;
      I := NewTermCh;
      Iteration := Rslt;
    end;

    { Process a picture group }
    function Group: TPXPictStatus;
    var
      Rslt: TPXPictStatus;
      TermCh: Byte;
    begin
      TermCh := CalcTerm;
      Inc(I);
      Rslt := Process(TermCh - 1);
      if not IsIncomplete(Rslt) then I := TermCh;
      Group := Rslt;
    end;

    function CheckComplete
    (Rslt: TPXPictStatus): TPXPictStatus;
    var
      J: Byte;
    begin
      J := I;
      if IsIncomplete(Rslt) then
      begin
        { Skip optional pieces }
        while True do
          case Pic^[J] of
            '[': ToGroupEnd(J);
            '*':
              if not IsNumber(Pic^[J + 1]) then
              begin
                Inc(J);
                ToGroupEnd(J);
              end
              else
                Break;
          else
            Break;
          end;

        if J = TermCh then Rslt := prAmbiguous;
      end;
      CheckComplete := Rslt;
    end;

    {Scan the input string}
    function Scan: TPXPictStatus;
    var
      Ch: Char;
      Rslt: TPXPictStatus;
    begin
      Scan := prError;
      Rslt := prEmpty;
      while (I <> TermCh) and (Pic^[I] <> ',') do
      begin
        if J > Length(Input) then
        begin
          Scan := CheckComplete(Rslt);
          Exit;
        end;

        Ch := Input[J];
        case Pic^[I] of
          '#': if not IsNumber(Ch) then Exit
               else Consume(Ch);
          '?': if not IsLetter(Ch) then Exit
               else Consume(Ch);
          '&': if not IsLetter(Ch) then Exit
               else Consume(UpCase(Ch));
          '!': Consume(UpCase(Ch));
          '@': Consume(Ch);
          '*':
            begin
              Rslt := Iteration;
              if not IsComplete(Rslt) then
              begin
                Scan := Rslt;
                Exit;
              end;
              if Rslt = prError then Rslt := prAmbiguous;
            end;
          '{':
            begin
              Rslt := Group;
              if not IsComplete(Rslt) or (I = TermCh) then
              begin
                Scan := Rslt;
                Exit;
              end;
            end;
          '[':
            begin
              Rslt := Group;
              if IsIncomplete(Rslt) or (I = TermCh) then
              begin
                Scan := Rslt;
                Exit;
              end;
              if Rslt = prError then Rslt := prAmbiguous;
            end;
        else
          if Pic^[I] = ';' then Inc(I);
          if UpCase(Pic^[I]) <> UpCase(Ch) then
            if Ch = ' ' then Ch := Pic^[I]
            else Exit;
          Consume(Pic^[I]);
        end;

        if Rslt = prAmbiguous then
          Rslt := prIncompNoFill
        else
          Rslt := prIncomplete;
      end;

      if Rslt = prIncompNoFill then
        Scan := prAmbiguous
      else
        Scan := prComplete;
    end;

  {Process}
  begin
    Incomp := False;
    OldI := I;
    OldJ := J;
    repeat
      Rslt := Scan;

      { Only accept completes if they make it farther in the input
        stream from the last incomplete }
      if (Rslt in [prComplete, prAmbiguous]) and InComp and (J < IncompJ) then
      begin
        Rslt := prIncomplete;
        J := IncompJ;
      end;

      if (Rslt = prError) or (Rslt = prIncomplete) then
      begin
        Process := Rslt;
        if not Incomp and (Rslt = prIncomplete) then
        begin
          Incomp := True;
          IncompI := I;
          IncompJ := J;
        end;
        I := OldI;
        J := OldJ;
        if not SkipToComma then
        begin
          if Incomp then
          begin
            Process := prIncomplete;
            I := IncompI;
            J := IncompJ;
          end;
          Exit;
        end;
        OldI := I;
      end;
    until (Rslt <> prError) and (Rslt <> prIncomplete);

    if (Rslt = prComplete) and Incomp then
      Process := prAmbiguous
    else
      Process := Rslt;
  end;

  {Preliminary syntax check}
  function SyntaxCheck: Boolean;
  var
    I: Integer;
    BrkLevel, BrcLevel: Integer;
  begin
    SyntaxCheck := False;

    if Pic^ = '' then Exit;

    if Pic^[Length(Pic^)] = ';' then Exit;
    if (Pic^[Length(Pic^)] = '*') and (Pic^[Length(Pic^) - 1] <> ';') then
      Exit;

    I := 1;
    BrkLevel := 0;
    BrcLevel := 0;
    while I <= Length(Pic^) do
    begin
      case Pic^[I] of
        '[': Inc(BrkLevel);
        ']': Dec(BrkLevel);
        '{': Inc(BrcLevel);
        '}': Dec(BrcLevel);
        ';': Inc(I);
      end;
      Inc(I);
    end;
    if (BrkLevel <> 0) or (BrcLevel <> 0) then Exit;

    SyntaxCheck := True;
  end;


{Eval}
begin
  Eval := prEmptyPicture;
  if Pic = Nil then Exit;

  Eval := prSyntax;
  if not SyntaxCheck then Exit;

  Eval := prEmpty;
  if (Input = '') then Exit;

  J := 1;
  I := 1;

  Rslt := Process(Length(Pic^) + 1);
  if (Rslt <> prError) and (Rslt <> prSyntax) and (J <= Length(Input)) then
    Rslt := prError;

  if (Rslt = prIncomplete) and AutoFill then
  begin
    Reprocess := False;
    while (I <= Length(Pic^)) and
      not IsSpecial(Pic^[I], '#?&!@*{}[],'#0) do
    begin
      if Pic^[I] = ';' then Inc(I);
      Input := Input + Pic^[I];
      Inc(I);
      Reprocess := True;
    end;
    J := 1;
    I := 1;
    if Reprocess then
      Rslt := Process(Length(Pic^) + 1)
  end;

  if Rslt = prAmbiguous then
    Eval := prComplete
  else if Rslt = prIncompNoFill then
    Eval := prIncomplete
  else
    Eval := Rslt;
end;

{
***********************************************************************
TPXCustomPictureEdit Methods
***********************************************************************
}

{
constructor Create;
}
constructor TPXCustomPictureEdit.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FPXPicture := TPXPictureValidator.Create;
end;

{
destructor Destroy;
}
destructor TPXCustomPictureEdit.Destroy;
begin
FPXPicture.Destroy;
inherited Destroy;
end;

{
procedure SetPXPicture
}
procedure TPXCustomPictureEdit.SetPXPicture(Value: TPXPictureValidator);
begin
FPXPicture.Assign(Value);
end;

{
procedure KeyPress
}
procedure TPXCustomPictureEdit.KeyPress(var Ch: Char);
var
  s:      string;
  len:    Word;
begin
inherited KeyPress(Ch);
if FPXPicture.HasPicture and (Ch in [#32..#255]) then
  begin
  s := PrepareText(Ch);
  FillText(s);
  Ch := #0;
  end;
end;


{
procedure PrepareText
}
function TPXCustomPictureEdit.PrepareText(const NewChar: Char): String;
var
  s: string;
begin
s := Text;
if (SelLength = 0) then
  insert(NewChar, s, SelStart + 1)
else
  begin
  delete(s, SelStart + 1, SelLength);
  insert(NewChar, s, SelStart + 1);
  end;
PrepareText := s;
end;

{
procedure FillText
}
function TPXCustomPictureEdit.FillText(var s: string): TPXPictStatus;
var
  len: Word;
  tne: TNotifyEvent;
  pr:  TPXPictStatus;
begin
pr := FPXPicture.Eval(s, FPXPicture.AutoFill);
len := Length(s);
if pr = prError then
  Error(s)
else
  begin
  Text := s;
  SelStart := len;
  end;
FillText := pr;
end;

{
procedure Error
}
procedure TPXCustomPictureEdit.Error(ErrStr: string);
var
  tne: TPXPErrorEvent;
begin
tne := OnError;
if Assigned(tne) then
  OnError(Self,ErrStr);
end;


{
function EditCanModify
}
function  TPXCustomPictureEdit.EditCanModify: Boolean;
begin
EditCanModify := True;
end;

{
***********************************************************************
TPXPictureEdit Methods
***********************************************************************
}

{
***********************************************************************
TPXDBPictureEdit Methods
***********************************************************************
}

{
constructor Create
}
constructor TPXDBPictureEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
  CalcTextMargin;
end;

{
destructor Destroy
}
destructor TPXDBPictureEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

{
procedure Notification
}
procedure TPXDBPictureEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

{
procedure KeyDown
}
procedure TPXDBPictureEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDataLink.Edit;
end;

{
procedure KeyPress
}
procedure TPXDBPictureEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
    not FDataLink.Field.IsValidChar(Key) then
  begin
    MessageBeep(0);
    Key := #0;
  end;
  case Key of
    ^H, ^V, ^X, #32..#255:
      FDataLink.Edit;
    #27:
      begin
        FDataLink.Reset;
        SelectAll;
        Key := #0;
      end;
  end;
end;

{
procedure EditCanModify
}
function TPXDBPictureEdit.EditCanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;

{
procedure Reset
}
procedure TPXDBPictureEdit.Reset;
begin
  FDataLink.Reset;
  SelectAll;
end;

{
procedure SetFocused
}
procedure TPXDBPictureEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
    begin
    FFocused := Value;
    if (FAlignment <> taLeftJustify) and not PXPicture.HasPicture then Invalidate;
    FDataLink.Reset;
    end;
end;

{
procedure Change
}
procedure TPXDBPictureEdit.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

{
procedure GetDataSource
}
function TPXDBPictureEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{
procedure SetDataSource
}
procedure TPXDBPictureEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{
procedure GetDataField
}
function TPXDBPictureEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

{
procedure SetDataField
}
procedure TPXDBPictureEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

{
procedure GetReadOnly
}
function TPXDBPictureEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

{
procedure SetReadOnly
}
procedure TPXDBPictureEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

{
procedure GetField
}
function TPXDBPictureEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

{
procedure DataChange
}
procedure TPXDBPictureEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    begin
    if FAlignment <> FDataLink.Field.Alignment then
      begin
      FAlignment := FDataLink.Field.Alignment;
      end;
    if FDataLink.Field.DataType = ftString then
      MaxLength := FDataLink.Field.Size
    else
      MaxLength := 0;
    if FDataLink.CanModify then
      Text := FDataLink.Field.Text;
    end
  else
    begin
    FAlignment := taLeftJustify;
    MaxLength := 0;
    end;
end;

{
procedure EditingChange
}
procedure TPXDBPictureEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
end;

{
procedure UpdateData
}
procedure TPXDBPictureEdit.UpdateData(Sender: TObject);
begin
  FDataLink.Field.Text := Text;
end;

{
procedure WMPaste
}
procedure TPXDBPictureEdit.WMPaste(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

{
procedure WMCut
}
procedure TPXDBPictureEdit.WMCut(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

{
procedure CMEnter
}
procedure TPXDBPictureEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;
end;

{
procedure CMExit
}
procedure TPXDBPictureEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
  SetCursor(0);
  DoExit;
end;

{
procedure WMPaint
}
procedure TPXDBPictureEdit.WMPaint(var Message: TWMPaint);
var
  Width, Indent, Left, I: Integer;
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
  S: string;
begin
  if (FAlignment = taLeftJustify) or FFocused then
  begin
    inherited;
    Exit;
  end;
{ Since edit controls do not handle justification unless multi-line (and
  then only poorly) we will draw right and center justify manually unless
  the edit has the focus. }
  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do
    begin
      R := ClientRect;
      if (BorderStyle = bsSingle) then
      begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        InflateRect(R, -1, -1);
      end;
      Brush.Color := Color;
      S := Text;
      if PasswordChar <> #0 then
        for I := 1 to Length(S) do
          S[I] := PasswordChar;
      Width := TextWidth(S);
      if BorderStyle = bsNone then Indent := 0 else Indent := FTextMargin;
      if FAlignment = taRightJustify then
        Left := R.Right - Width - Indent else
        Left := (R.Left + R.Right - Width) div 2;
      TextRect(R, Left, Indent, S);
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0 then EndPaint(Handle, PS);
  end;
end;

{
procedure CMFontChanged
}
procedure TPXDBPictureEdit.CMFontChanged(var Message: TMessage);
begin
  inherited;
  CalcTextMargin;
end;

{
procedure CalcTextMargin
}
procedure TPXDBPictureEdit.CalcTextMargin;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  DC := GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  I := SysMetrics.tmHeight;
  if I > Metrics.tmHeight then I := Metrics.tmHeight;
  FTextMargin := I div 4;
end;

end.
