Unit StrOps;

{ Here are some miscellaneous procs I wrote a long time ago
  for TP, optimized for size and speed.
  Some of these procs can be faster, but I didn't think that
  the size increase warranted the slight gains in speed.

  I've been using these procs for years and am sure you will
  find no bugs. The entire unit uses a total of 364 bytes of
  code and the procs are very fast.

  To use the procs, simply add strops to the Units list.

  If you have any comments or questions, feel free to contact
  me on CompuServe.

	  Don Strenczewilk CIS 72617,132
}

(*******************************************************************)
			Interface
(*******************************************************************)

Procedure FillZero(VAR Target; Size : Word);
{ Equivalent to: Fillchar(Target, Size, 0) }

function streq(const s1,s2 : string) : boolean;
 { returns true if s1 = s2. }

function BytesAreEqual(a1, a2 : pointer; size : integer) : boolean;
{ Compares two structures of the same size for equality }
{ Call with: If BytesAreEqual(@Va1, @Var2, Sizeof(Var1)) }

Function CCount(Cha : Char; const S : String) : Integer;
{ Returns number of occurences of Ch in S.}

Procedure Sp2Zero( Var Str : String );
{ converts all occurrences of ' ' to '0' }

Function TrimL(const S : String) : String;
{ Trims Leading chars <= ' ' }

Function TrimT(const S : String) : String;
{ Trims Trailing chars <= ' ' }

Procedure VTrimT(VAR S : String);
{ Trims Trailing chars <= ' ' }

Function TrimB(const S : String) : String;
{ Trims Leading/Trailing chars <= ' ' }

Procedure VTrimB(VAR S : String);
{ Trims Leading/Trailing chars <= ' ' }

Function UpCaseSt(const S : String) : String;
{ Converts S to upper case }

Procedure VUpCaseSt(VAR S : String);
{ Converts S to upper case }

Function LoCaseSt(const S : String) : String;
{ Converts S to lower case }

Procedure VLoCaseSt(VAR S : String);
{ Converts S to lower case }



(*******************************************************************)
		Implementation
(*******************************************************************)

function streq(const s1,s2 : string) : boolean; assembler;
asm	push	ds
	xor	al, al
	lds	si, s1
	les	di, s2
	cld
	cmpsb
	 jne	@9
	mov	cl, byte ptr [si-1]
	xor	ch, ch
	repe	cmpsb
	 jne	@9
	inc	al
@9:	pop	ds
end;


{-------------------------------------------------------------------}
Function CCount(Cha : Char; const S : String) : Integer; assembler;
asm	les  di, [s]		{;es:di => s}
	xor  cx, cx
	mov  dx, cx
	mov  cl, [es:di]	{;cl = length(s)}
	inc  di
	mov  al, [cha]
	cld
@@Next: jcxz @@Done
	repne scasb
	 jne  @@Done
	inc  dl
	jmp @@Next
@@Done:	mov  ax, dx
END;
{-------------------------------------------------------------------}

Procedure Sp2Zero( Var Str : String ); assembler;
asm	les  di, [Str]
	mov  cl, [es:di]	{; cx = length(Strg)}
	xor  ch, ch
	inc  di
	mov  ax, 3020h		{; ah = '0', al = ' '}
	cld
	jmp @Next
@Found:	mov  [byte ptr es:di-1], ah
	 jcxz @Done
@Next:	repne scasb
	 je  @Found
@Done:
END;

{-------------------------------------------------------------------}
{$L PTRIM.OBJ}
Function TrimL(const S : String) : String;	External;
Function TrimT(const S : String) : String;	External;
Procedure VTrimT(VAR S : String);	External;
Function TrimB(const S : String) : String;	External;
Procedure VTrimB(VAR S : String);	External;
Function UpCaseSt(const S : String) : String;	External;
Procedure VUpCaseSt(VAR S : String);	External;
Function LoCaseSt(const S : String) : String;	External;
Procedure VLoCaseSt(VAR S : String);	External;

{-------------------------------------------------------------------}

Procedure FillZero(VAR Target; Size : Word); assembler;
asm	xor  ax, ax
	cld
	les  di, Target
	mov  cx, Size
	shr  cx, 1
	rep  stosw
	 jnc  @SWords
	stosb
@SWords:
END;


function BytesAreEqual(a1, a2 : pointer; size : integer) : boolean; assembler;
asm	mov	bx, ds
	lds	si, a1
	les	di, a2
	mov	cx, size
	xor	ax, ax
	repe	cmpsb
	 jne	@9
	inc	ax
@9:	mov	ds, bx
end;



{ Unit Initialization }
end.